@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2017-2020 GEM Foundation
+# Copyright (C) 2017-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -18,6 +18,9 @@
 
 import os
 import sys
+import numpy
+import scipy
+import pandas
 import configparser
 
 # disable OpenBLAS threads before the first numpy import
@@ -26,9 +29,15 @@ os.environ['OPENBLAS_NUM_THREADS'] = '1'
 from openquake.baselib.general import git_suffix  # noqa: E402
 
 # the version is managed by packager.sh with a sed
-__version__ = '3.9.0'
+__version__ = '3.11.0'
 __version__ += git_suffix(__file__)
 
+version = dict(engine=__version__,
+               python='%d.%d' % sys.version_info[:2],
+               numpy=numpy.__version__,
+               scipy=scipy.__version__,
+               pandas=pandas.__version__)
+
 
 class InvalidFile(Exception):
     """Raised from custom file validators"""
@@ -91,27 +100,33 @@ def read(*paths, **validators):
     for section in parser.sections():
         config[section] = sec = DotDict(parser.items(section))
         for k, v in sec.items():
-            sec[k] = validators.get(k, lambda x: x)(v)
+            try:
+                sec[k] = validators.get(k, lambda x: x)(v)
+            except ValueError as err:
+                raise ValueError('%s for %s in %s' % (err, k, found[-1]))
 
 
 config.read = read
 
 
-def boolean(flag):
+def positiveint(flag):
     """
-    Convert string in boolean
+    Convert string into integer
     """
     s = flag.lower()
     if s in ('1', 'yes', 'true'):
-        return True
+        return 1
     elif s in ('0', 'no', 'false'):
-        return False
-    raise ValueError('Unknown flag %r' % s)
+        return 0
+    i = int(s)
+    if i < 0:
+        raise ValueError('Invalid %r' % s)
+    return i
 
 
-config.read(soft_mem_limit=int, hard_mem_limit=int, port=int,
-            multi_user=boolean,
-            serialize_jobs=boolean, strict=boolean, code=exec)
+config.read(limit=int, soft_mem_limit=int, hard_mem_limit=int, port=int,
+            multi_user=positiveint, serialize_jobs=positiveint,
+            strict=positiveint, code=exec)
 
 if config.directory.custom_tmp:
     os.environ['TMPDIR'] = config.directory.custom_tmp
