@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright (C) 2015-2020 GEM Foundation
+# Copyright (C) 2015-2021 GEM Foundation
 
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -18,6 +18,7 @@
 
 import os
 import time
+import pickle
 import getpass
 import operator
 import itertools
@@ -46,7 +47,7 @@ def init_performance(hdf5file, swmr=False):
     :param hdf5file: file name of hdf5.File instance
     """
     fname = isinstance(hdf5file, str)
-    h5 = hdf5.File(hdf5file) if fname else hdf5file
+    h5 = hdf5.File(hdf5file, 'a') if fname else hdf5file
     if 'performance_data' not in h5:
         hdf5.create(h5, 'performance_data', perf_dt)
     if 'task_info' not in h5:
@@ -80,8 +81,12 @@ def performance_view(dstore):
             mem = max(mem, rec['memory_mb'])
         out.append((operation, time, mem, counts))
     out.sort(key=operator.itemgetter(1), reverse=True)  # sort by time
-    operation = ('calc_%d' % dstore.calc_id if hasattr(dstore, 'calc_id')
-                 else 'operation')
+    mems = dstore['task_info']['mem_gb']
+    maxmem = ', maxmem=%.1f GB' % mems.max() if len(mems) else ''
+    if hasattr(dstore, 'calc_id'):
+        operation = 'calc_%d%s' % (dstore.calc_id, maxmem)
+    else:
+        operation = 'operation'
     dtlist = [(operation, perf_dt['operation'])]
     dtlist.extend((n, perf_dt[n]) for n in perf_dt.names[1:-1])
     return numpy.array(out, dtlist)
@@ -264,6 +269,35 @@ class Monitor(object):
         vars(new).update(kw)
         return new
 
+    def save(self, key, obj):
+        """
+        :param key: key in the _tmp.hdf5 file
+        :param obj: big object to store in pickle format
+        :returns: True is saved, False if not because the key was taken
+        """
+        tmp = self.filename[:-5] + '_tmp.hdf5'
+        f = hdf5.File(tmp, 'a') if os.path.exists(tmp) else hdf5.File(tmp, 'w')
+        with f:
+            if key in f:  # already saved
+                return False
+            if isinstance(obj, numpy.ndarray):
+                f[key] = obj
+            else:
+                f[key] = pickle.dumps(obj, protocol=pickle.HIGHEST_PROTOCOL)
+        return True
+
+    def read(self, key):
+        """
+        :param key: key in the _tmp.hdf5 file
+        :return: unpickled object
+        """
+        tmp = self.filename[:-5] + '_tmp.hdf5'
+        with hdf5.File(tmp, 'r') as f:
+            data = f[key][()]
+            if data.shape:
+                return data
+            return pickle.loads(data)
+
     def __repr__(self):
         calc_id = ' #%s ' % self.calc_id if self.calc_id else ' '
         msg = '%s%s%s[%s]' % (self.__class__.__name__, calc_id,
