@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2020 GEM Foundation
+# Copyright (C) 2014-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -15,13 +15,14 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake. If not, see <http://www.gnu.org/licenses/>.
+import json
+import logging
 import itertools
 import collections
 import numpy
+import pandas
 
 from openquake.baselib import hdf5
-from openquake.baselib.python3compat import decode
-from openquake.baselib.general import group_array
 from openquake.hazardlib.stats import compute_stats2
 from openquake.risklib import scientific
 from openquake.calculators.extract import (
@@ -39,22 +40,6 @@ U32 = numpy.uint32
 stat_dt = numpy.dtype([('mean', F32), ('stddev', F32)])
 
 
-def add_columns(table, **columns):
-    """
-    :param table: a list of rows, with the first row being an header
-    :param columns: a dictionary of functions producing the dynamic columns
-    """
-    fields, *rows = table
-    Ntuple = collections.namedtuple('nt', fields)
-    newtable = [fields + tuple(columns)]
-    for rec in itertools.starmap(Ntuple, rows):
-        newrow = list(rec)
-        for col in columns:
-            newrow.append(columns[col](rec))
-        newtable.append(newrow)
-    return newtable
-
-
 def get_rup_data(ebruptures):
     dic = {}
     for ebr in ebruptures:
@@ -70,40 +55,78 @@ def tag2idx(tags):
 
 
 # this is used by event_based_risk and ebrisk
-@export.add(('agg_curves-rlzs', 'csv'), ('agg_curves-stats', 'csv'),
-            ('tot_curves-rlzs', 'csv'), ('tot_curves-stats', 'csv'))
+@export.add(('agg_curves-rlzs', 'csv'), ('agg_curves-stats', 'csv'))
 def export_agg_curve_rlzs(ekey, dstore):
     oq = dstore['oqparam']
-    assetcol = dstore['assetcol']
-    if ekey[0].startswith('agg_'):
-        aggregate_by = oq.aggregate_by
-    else:  # tot_curves
-        aggregate_by = []
-
-    name = '_'.join(['agg'] + aggregate_by)
-    aggvalue = dstore['exposed_values/' + name][()]
-
-    lti = tag2idx(oq.loss_names)
-    tagi = {tagname: tag2idx(getattr(assetcol.tagcol, tagname))
-            for tagname in aggregate_by}
-
-    def get_loss_ratio(rec):
-        idxs = tuple(tagi[tagname][getattr(rec, tagname)] - 1
-                     for tagname in aggregate_by) + (lti[rec.loss_types],)
-        return rec.loss_value / aggvalue[idxs]
-
-    # shape (T1, T2, ..., L)
+    lnames = numpy.array(oq.loss_names)
+    if oq.aggregate_by:
+        agg_keys = dstore['agg_keys'][:]
+    agg_tags = {}
+    for tagname in oq.aggregate_by:
+        agg_tags[tagname] = numpy.concatenate([agg_keys[tagname], ['*total*']])
+    aggvalue = dstore['agg_values'][()]  # shape (K+1, L)
     md = dstore.metadata
-    md.update(dict(
-        kind=ekey[0], risk_investigation_time=oq.risk_investigation_time))
-    fname = dstore.export_path('%s.%s' % ekey)
+    md['risk_investigation_time'] = oq.risk_investigation_time
     writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
-    aw = hdf5.ArrayWrapper.from_(dstore[ekey[0]], 'loss_value')
-    table = add_columns(
-        aw.to_table(), loss_ratio=get_loss_ratio,
-        annual_frequency_of_exceedence=lambda rec: 1 / rec.return_periods)
-    table[0] = [c[:-1] if c.endswith('s') else c for c in table[0]]
-    writer.save(table, fname, comment=md)
+    descr = hdf5.get_shape_descr(dstore[ekey[0]].attrs['json'])
+    name, suffix = ekey[0].split('-')
+    rlzs_or_stats = descr[suffix[:-1]]
+    aw = hdf5.ArrayWrapper(dstore[ekey[0]], descr, ('loss_value',))
+    dataf = aw.to_dframe().set_index(suffix[:-1])
+    for r, ros in enumerate(rlzs_or_stats):
+        md['kind'] = f'{name}-' + (
+            ros if isinstance(ros, str) else 'rlz-%03d' % ros)
+        try:
+            df = dataf[dataf.index == ros]
+        except KeyError:
+            logging.warning('No data for %s', md['kind'])
+            continue
+        dic = {col: df[col].to_numpy() for col in dataf.columns}
+        dic['loss_type'] = lnames[dic['lti']]
+        for tagname in oq.aggregate_by:
+            dic[tagname] = agg_tags[tagname][dic['agg_id']]
+        dic['loss_ratio'] = dic['loss_value'] / aggvalue[
+            dic['agg_id'], dic.pop('lti')]
+        dic['annual_frequency_of_exceedence'] = 1 / dic['return_period']
+        del dic['agg_id']
+        dest = dstore.build_fname(md['kind'], '', 'csv')
+        writer.save(pandas.DataFrame(dic), dest, comment=md)
+    return writer.getsaved()
+
+
+# this is used by ebrisk
+@export.add(('agg_losses-rlzs', 'csv'), ('agg_losses-stats', 'csv'))
+def export_agg_losses(ekey, dstore):
+    """
+    :param ekey: export key, i.e. a pair (datastore key, fmt)
+    :param dstore: datastore object
+    """
+    dskey = ekey[0]
+    oq = dstore['oqparam']
+    aggregate_by = oq.aggregate_by if dskey.startswith('agg_') else []
+    name, value, rlzs_or_stats = _get_data(dstore, dskey, oq.hazard_stats())
+    # value has shape (K, R, L)
+    writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
+    tagcol = dstore['assetcol/tagcol']
+    aggtags = list(tagcol.get_aggkey(aggregate_by).values())
+    aggtags.append(('*total*',) * len(aggregate_by))
+    expvalue = dstore['agg_values'][()]  # shape (K+1, L)
+    tagnames = tuple(aggregate_by)
+    header = ('loss_type',) + tagnames + (
+        'loss_value', 'exposed_value', 'loss_ratio')
+    md = dstore.metadata
+    md.update(dict(investigation_time=oq.investigation_time,
+              risk_investigation_time=oq.risk_investigation_time))
+    for r, ros in enumerate(rlzs_or_stats):
+        ros = ros if isinstance(ros, str) else 'rlz-%03d' % ros
+        rows = []
+        for (k, l), loss in numpy.ndenumerate(value[:, r]):
+            if loss:  # many tag combinations are missing
+                evalue = expvalue[k, l]
+                row = aggtags[k] + (loss, evalue, loss / evalue)
+                rows.append((oq.loss_names[l],) + row)
+        dest = dstore.build_fname(name, ros, 'csv')
+        writer.save(rows, dest, header, comment=md)
     return writer.getsaved()
 
 
@@ -112,21 +135,21 @@ def _get_data(dstore, dskey, stats):
     if kind == 'stats':
         weights = dstore['weights'][()]
         if dskey in set(dstore):  # precomputed
-            tags = [decode(s) for s in dstore.get_attr(dskey, 'stats')]
-            statfuncs = [stats[tag] for tag in tags]
+            rlzs_or_stats = list(stats)
+            statfuncs = [stats[ros] for ros in stats]
             value = dstore[dskey][()]  # shape (A, S, LI)
-        else:  # computed on the fly
-            tags, statfuncs = zip(*stats.items())
+        else:  # compute on the fly
+            rlzs_or_stats, statfuncs = zip(*stats.items())
             value = compute_stats2(
                 dstore[name + '-rlzs'][()], statfuncs, weights)
     else:  # rlzs
         value = dstore[dskey][()]  # shape (A, R, LI)
         R = value.shape[1]
-        tags = ['rlz-%03d' % r for r in range(R)]
-    return name, value, tags
+        rlzs_or_stats = ['rlz-%03d' % r for r in range(R)]
+    return name, value, rlzs_or_stats
 
 
-# this is used by event_based_risk and classical_risk
+# this is used by event_based_risk, classical_risk and scenario_risk
 @export.add(('avg_losses-rlzs', 'csv'), ('avg_losses-stats', 'csv'))
 def export_avg_losses(ekey, dstore):
     """
@@ -136,14 +159,14 @@ def export_avg_losses(ekey, dstore):
     dskey = ekey[0]
     oq = dstore['oqparam']
     dt = [(ln, F32) for ln in oq.loss_names]
-    name, value, tags = _get_data(dstore, dskey, oq.hazard_stats())
+    name, value, rlzs_or_stats = _get_data(dstore, dskey, oq.hazard_stats())
     writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
     assets = get_assets(dstore)
     md = dstore.metadata
     md.update(dict(investigation_time=oq.investigation_time,
                    risk_investigation_time=oq.risk_investigation_time))
-    for tag, values in zip(tags, value.transpose(1, 0, 2)):
-        dest = dstore.build_fname(name, tag, 'csv')
+    for ros, values in zip(rlzs_or_stats, value.transpose(1, 0, 2)):
+        dest = dstore.build_fname(name, ros, 'csv')
         array = numpy.zeros(len(values), dt)
         for li, ln in enumerate(oq.loss_names):
             array[ln] = values[:, li]
@@ -152,69 +175,26 @@ def export_avg_losses(ekey, dstore):
     return writer.getsaved()
 
 
-# this is used by ebrisk
-@export.add(('agg_losses-rlzs', 'csv'), ('agg_losses-stats', 'csv'),
-            ('tot_losses-rlzs', 'csv'), ('tot_losses-stats', 'csv'))
-def export_agg_losses(ekey, dstore):
-    """
-    :param ekey: export key, i.e. a pair (datastore key, fmt)
-    :param dstore: datastore object
-    """
-    dskey = ekey[0]
-    oq = dstore['oqparam']
-    aggregate_by = oq.aggregate_by if dskey.startswith('agg_') else []
-    name, value, tags = _get_data(dstore, dskey, oq.hazard_stats())
-    writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
-    assetcol = dstore['assetcol']
-    aggname = '_'.join(['agg'] + aggregate_by)
-    expvalue = dstore['exposed_values/' + aggname][()]
-    # shape (T1, T2, ..., L)
-    tagnames = tuple(aggregate_by)
-    header = ('loss_type',) + tagnames + (
-        'loss_value', 'exposed_value', 'loss_ratio')
-    md = dstore.metadata
-    md.update(dict(investigation_time=oq.investigation_time,
-              risk_investigation_time=oq.risk_investigation_time))
-    for r, tag in enumerate(tags):
-        rows = []
-        for multi_idx, loss in numpy.ndenumerate(value[:, r]):
-            l, *tagidxs = multi_idx
-            evalue = expvalue[tuple(tagidxs) + (l,)]
-            row = assetcol.tagcol.get_tagvalues(tagnames, tagidxs) + (
-                loss, evalue, loss / evalue)
-            rows.append((oq.loss_names[l],) + row)
-        dest = dstore.build_fname(name, tag, 'csv')
-        writer.save(rows, dest, header, comment=md)
-    return writer.getsaved()
-
-
-# this is used by scenario_risk
-@export.add(('losses_by_asset', 'csv'))
-def export_losses_by_asset(ekey, dstore):
+@export.add(('src_loss_table', 'csv'))
+def export_src_loss_table(ekey, dstore):
     """
     :param ekey: export key, i.e. a pair (datastore key, fmt)
     :param dstore: datastore object
     """
     oq = dstore['oqparam']
-    loss_dt = oq.loss_dt(stat_dt)
-    losses_by_asset = dstore[ekey[0]][()]
-    rlzs = dstore['full_lt'].get_realizations()
-    assets = get_assets(dstore)
-    writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
     md = dstore.metadata
     md.update(dict(investigation_time=oq.investigation_time,
                    risk_investigation_time=oq.risk_investigation_time))
-    for rlz in rlzs:
-        losses = losses_by_asset[:, rlz.ordinal]
-        dest = dstore.build_fname('losses_by_asset', rlz, 'csv')
-        data = compose_arrays(assets, losses.copy().view(loss_dt)[:, 0])
-        writer.save(data, dest, comment=md, renamedict=dict(id='asset_id'))
+    aw = hdf5.ArrayWrapper.from_(dstore['src_loss_table'], 'loss_value')
+    dest = dstore.build_fname('src_loss_table', '', 'csv')
+    writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
+    writer.save(aw.to_dframe(), dest, comment=md)
     return writer.getsaved()
 
 
 # this is used by scenario_risk, event_based_risk and ebrisk
-@export.add(('losses_by_event', 'csv'))
-def export_losses_by_event(ekey, dstore):
+@export.add(('agg_loss_table', 'csv'))
+def export_agg_loss_table(ekey, dstore):
     """
     :param ekey: export key, i.e. a pair (datastore key, fmt)
     :param dstore: datastore object
@@ -227,20 +207,20 @@ def export_losses_by_event(ekey, dstore):
         md.update(dict(investigation_time=oq.investigation_time,
                        risk_investigation_time=oq.risk_investigation_time))
     events = dstore['events'][()]
-    columns = dict(rlz_id=lambda rec: events[rec.event_id]['rlz_id'])
+    try:
+        K = dstore.get_attr('agg_loss_table', 'K', 0)
+        df = dstore.read_df('agg_loss_table', 'agg_id', dict(agg_id=K))
+    except KeyError:  # scenario_damage + consequences
+        df = dstore.read_df('losses_by_event')
+        ren = {'loss_%d' % li: ln for li, ln in enumerate(oq.loss_names)}
+        df.rename(columns=ren, inplace=True)
+    evs = events[df.event_id.to_numpy()]
+    df['rlz_id'] = evs['rlz_id']
     if oq.investigation_time:  # not scenario
-        columns['rup_id'] = lambda rec: events[rec.event_id]['rup_id']
-        columns['year'] = lambda rec: events[rec.event_id]['year']
-    lbe = dstore['losses_by_event'][()]
-    lbe.sort(order='event_id')
-    dic = dict(shape_descr=['event_id'])
-    dic['event_id'] = list(lbe['event_id'])
-    # example (0, 1, 2, 3) -> (0, 2, 3, 1)
-    axis = [0] + list(range(2, len(lbe['loss'].shape))) + [1]
-    data = lbe['loss'].transpose(axis)  # shape (E, T..., L)
-    aw = hdf5.ArrayWrapper(data, dic, oq.loss_names)
-    table = add_columns(aw.to_table(), **columns)
-    writer.save(table, dest, comment=md)
+        df['rup_id'] = evs['rup_id']
+        df['year'] = evs['year']
+    df.sort_values('event_id', inplace=True)
+    writer.save(df, dest, comment=md)
     return writer.getsaved()
 
 
@@ -254,7 +234,7 @@ def _compact(array):
     return array.view(numpy.dtype(lst)).reshape(a)
 
 
-# this is used by classical_risk and event_based_risk
+# this is used by classical_risk
 @export.add(('loss_curves-rlzs', 'csv'), ('loss_curves-stats', 'csv'),
             ('loss_curves', 'csv'))
 def export_loss_curves(ekey, dstore):
@@ -273,17 +253,17 @@ def export_loss_maps_csv(ekey, dstore):
     value = get_loss_maps(dstore, kind)
     oq = dstore['oqparam']
     if kind == 'rlzs':
-        tags = dstore['full_lt'].get_realizations()
+        rlzs_or_stats = dstore['full_lt'].get_realizations()
     else:
-        tags = oq.hazard_stats()
+        rlzs_or_stats = oq.hazard_stats()
     writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
     md = dstore.metadata
-    for i, tag in enumerate(tags):
-        if hasattr(tag, 'ordinal'):  # is a realization
-            tag = 'rlz-%d' % tag.ordinal
-        fname = dstore.build_fname('loss_maps', tag, ekey[1])
+    for i, ros in enumerate(rlzs_or_stats):
+        if hasattr(ros, 'ordinal'):  # is a realization
+            ros = 'rlz-%d' % ros.ordinal
+        fname = dstore.build_fname('loss_maps', ros, ekey[1])
         md.update(
-            dict(kind=tag, risk_investigation_time=oq.risk_investigation_time))
+            dict(kind=ros, risk_investigation_time=oq.risk_investigation_time))
         writer.save(compose_arrays(assets, value[:, i]), fname, comment=md,
                     renamedict=dict(id='asset_id'))
     return writer.getsaved()
@@ -297,68 +277,57 @@ def export_loss_maps_npz(ekey, dstore):
     value = get_loss_maps(dstore, kind)
     R = dstore['full_lt'].get_num_rlzs()
     if kind == 'rlzs':
-        tags = ['rlz-%03d' % r for r in range(R)]
+        rlzs_or_stats = ['rlz-%03d' % r for r in range(R)]
     else:
         oq = dstore['oqparam']
-        tags = oq.hazard_stats()
+        rlzs_or_stats = oq.hazard_stats()
     fname = dstore.export_path('%s.%s' % ekey)
     dic = {}
-    for i, tag in enumerate(tags):
-        dic[tag] = compose_arrays(assets, value[:, i])
+    for i, ros in enumerate(rlzs_or_stats):
+        dic[ros] = compose_arrays(assets, value[:, i])
     savez(fname, **dic)
     return [fname]
 
 
-@export.add(('damages-rlzs', 'csv'), ('damages-stats', 'csv'))
-def export_damages_csv(ekey, dstore):
-    rlzs = dstore['full_lt'].get_realizations()
-    oq = dstore['oqparam']
-    loss_types = oq.loss_dt().names
-    assets = get_assets(dstore)
-    value = dstore[ekey[0]][()]  # matrix N x R x LI or T x R x LI
-    writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
-    if ekey[0].endswith('stats'):
-        tags = oq.hazard_stats()
-    else:
-        tags = ['rlz-%03d' % r for r in range(len(rlzs))]
-    for lti, lt in enumerate(loss_types):
-        for tag, values in zip(tags, value[:, :, lti].T):
-            fname = dstore.build_fname('damages-%s' % lt, tag, ekey[1])
-            writer.save(compose_arrays(assets, values), fname,
-                        renamedict=dict(id='asset_id'))
-    return writer.getsaved()
-
-
 def modal_damage_array(data, damage_dt):
     # determine the damage state with the highest probability
-    A, L, MS, D = data.shape
+    A, L, D = data.shape
     dmgstate = damage_dt['structural'].names
     arr = numpy.zeros(A, [('modal-ds-' + lt, hdf5.vstr)
                           for lt in damage_dt.names])
-    for l, loss_type in enumerate(damage_dt.names):
-        arr['modal-ds-' + loss_type] = [dmgstate[data[a, l, 0].argmax()]
+    for li, loss_type in enumerate(damage_dt.names):
+        arr['modal-ds-' + loss_type] = [dmgstate[data[a, li].argmax()]
                                         for a in range(A)]
     return arr
 
 
-@export.add(('dmg_by_asset', 'csv'))
-def export_dmg_by_asset_csv(ekey, dstore):
-    E = len(dstore['events'])
+@export.add(('damages-rlzs', 'csv'), ('damages-stats', 'csv'))
+def export_damages_csv(ekey, dstore):
     oq = dstore['oqparam']
-    dmg_dt = build_damage_dt(dstore, mean_std=False)
-    damage_dt = build_damage_dt(dstore, mean_std=E > 1)
+    dmg_dt = build_damage_dt(dstore)
     rlzs = dstore['full_lt'].get_realizations()
     data = dstore[ekey[0]]
     writer = writers.CsvWriter(fmt='%.6E')
     assets = get_assets(dstore)
-    for rlz in rlzs:
+    md = dstore.metadata
+    if oq.investigation_time:
+        md.update(dict(investigation_time=oq.investigation_time,
+                       risk_investigation_time=oq.risk_investigation_time))
+    if ekey[0].endswith('stats'):
+        rlzs_or_stats = oq.hazard_stats()
+    else:
+        rlzs_or_stats = ['rlz-%03d' % r for r in range(len(rlzs))]
+    name = ekey[0].split('-')[0]
+    if oq.calculation_mode != 'classical_damage':
+        name = 'avg_' + name
+    for i, ros in enumerate(rlzs_or_stats):
         if oq.modal_damage_state:
-            dmg_by_asset = modal_damage_array(data[:, rlz.ordinal], dmg_dt)
+            damages = modal_damage_array(data[:, i], dmg_dt)
         else:
-            dmg_by_asset = build_damage_array(data[:, rlz.ordinal], damage_dt)
-        fname = dstore.build_fname(ekey[0], rlz, ekey[1])
-        writer.save(compose_arrays(assets, dmg_by_asset), fname,
-                    renamedict=dict(id='asset_id'))
+            damages = build_damage_array(data[:, i], dmg_dt)
+        fname = dstore.build_fname(name, ros, ekey[1])
+        writer.save(compose_arrays(assets, damages), fname,
+                    comment=md, renamedict=dict(id='asset_id'))
     return writer.getsaved()
 
 
@@ -368,24 +337,23 @@ def export_dmg_by_event(ekey, dstore):
     :param ekey: export key, i.e. a pair (datastore key, fmt)
     :param dstore: datastore object
     """
-    damage_dt = build_damage_dt(dstore, mean_std=False)
+    damage_dt = build_damage_dt(dstore)
     dt_list = [('event_id', U32), ('rlz_id', U16)] + [
         (f, damage_dt.fields[f][0]) for f in damage_dt.names]
     dmg_by_event = dstore[ekey[0]][()]  # shape E, L, D
-    events_by_rlz = group_array(dstore['events'], 'rlz_id')
-    writer = writers.CsvWriter(fmt='%d')
+    events = dstore['events'][()]
+    writer = writers.CsvWriter(fmt='%g')
     fname = dstore.build_fname('dmg_by_event', '', 'csv')
     writer.save(numpy.zeros(0, dt_list), fname)
-    with open(fname, 'ab') as dest:
-        for rlz, events in events_by_rlz.items():
-            data = numpy.array(  # shape (E, L, D)
-                [dmg_by_event[eid] for eid in events['id']])
-            arr = numpy.zeros(len(data), dt_list)
-            arr['event_id'] = events['id']
-            arr['rlz_id'] = events['rlz_id']
+    with open(fname, 'a') as dest:
+        for rlz_id in numpy.unique(events['rlz_id']):
+            ok, = numpy.where(events['rlz_id'] == rlz_id)
+            arr = numpy.zeros(len(ok), dt_list)
+            arr['event_id'] = events['id'][ok]
+            arr['rlz_id'] = rlz_id
             for l, loss_type in enumerate(damage_dt.names):
                 for d, dmg_state in enumerate(damage_dt[loss_type].names):
-                    arr[loss_type][dmg_state] = data[:, l, d]
+                    arr[loss_type][dmg_state] = dmg_by_event[ok, l, d]
             writer.save_block(arr, dest)
     return [fname]
 
@@ -483,13 +451,13 @@ def export_bcr_map(ekey, dstore):
     bcr_data = dstore[ekey[0]]
     N, R = bcr_data.shape
     if ekey[0].endswith('stats'):
-        tags = oq.hazard_stats()
+        rlzs_or_stats = oq.hazard_stats()
     else:
-        tags = ['rlz-%03d' % r for r in range(R)]
+        rlzs_or_stats = ['rlz-%03d' % r for r in range(R)]
     fnames = []
     writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
-    for t, tag in enumerate(tags):
-        path = dstore.build_fname('bcr', tag, 'csv')
+    for t, ros in enumerate(rlzs_or_stats):
+        path = dstore.build_fname('bcr', ros, 'csv')
         writer.save(compose_arrays(assets, bcr_data[:, t]), path,
                     renamedict=dict(id='asset_id'))
         fnames.append(path)
@@ -508,7 +476,7 @@ def export_aggregate_by_csv(ekey, dstore):
     writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
     path = '%s.%s' % (sanitize(ekey[0]), ekey[1])
     fname = dstore.export_path(path)
-    writer.save(aw.to_table(), fname)
+    writer.save(aw.to_dframe(), fname)
     fnames.append(fname)
     return fnames
 
@@ -522,15 +490,15 @@ def export_asset_risk_csv(ekey, dstore):
     writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
     path = '%s.%s' % (sanitize(ekey[0]), ekey[1])
     fname = dstore.export_path(path)
-    md = extract(dstore, 'exposure_metadata')
-    tostr = {'taxonomy': md.taxonomy}
-    for tagname in md.tagnames:
-        tostr[tagname] = getattr(md, tagname)
-    tagnames = sorted(set(md.tagnames) - {'id'})
+    md = json.loads(extract(dstore, 'exposure_metadata').json)
+    tostr = {'taxonomy': md['taxonomy']}
+    for tagname in md['tagnames']:
+        tostr[tagname] = md[tagname]
+    tagnames = sorted(set(md['tagnames']) - {'id'})
     arr = extract(dstore, 'asset_risk').array
     rows = []
     lossnames = sorted(name for name in arr.dtype.names if 'loss' in name)
-    expnames = [name for name in arr.dtype.names if name not in md.tagnames
+    expnames = [name for name in arr.dtype.names if name not in md['tagnames']
                 and 'loss' not in name and name not in 'lon lat']
     colnames = tagnames + ['lon', 'lat'] + expnames + lossnames
     # sanity check
@@ -540,7 +508,7 @@ def export_asset_risk_csv(ekey, dstore):
         for name in colnames:
             value = rec[name]
             try:
-                row.append('"%s"' % tostr[name][value])
+                row.append(tostr[name][value])
             except KeyError:
                 row.append(value)
         rows.append(row)
