@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2020 GEM Foundation
+# Copyright (C) 2018-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -17,12 +17,11 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
 import numpy
-from openquake.baselib import sap, performance
+from openquake.baselib import performance
 from openquake.hazardlib.shakemap import download_array
 
 
-@sap.script
-def download_shakemap(id):
+def main(id):
     """
     Example of usage: utils/shakemap usp000fjta
     """
@@ -31,6 +30,3 @@ def download_shakemap(id):
         numpy.save(dest, download_array(id))
     print(mon)
     print('Saved %s' % dest)
-
-
-download_shakemap.arg('id', 'USGS Shakemap ID')
