@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2020 GEM Foundation
+# Copyright (C) 2018-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -17,15 +17,14 @@
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
 import sys
 import logging
-from openquake.baselib import sap, datastore
+from openquake.baselib import datastore
 from openquake.commonlib import logs
 from openquake.calculators.extract import WebExtractor
 from openquake.engine import engine
 from openquake.server import dbserver
 
 
-@sap.script
-def importcalc(calc_id):
+def main(calc_id):
     """
     Import a remote calculation into the local database. server, username
     and password must be specified in an openquake.cfg file.
@@ -36,7 +35,6 @@ def importcalc(calc_id):
     try:
         calc_id = int(calc_id)
     except ValueError:  # assume calc_id is a pathname
-        status = 'complete'
         remote = False
     else:
         remote = True
@@ -46,7 +44,6 @@ def importcalc(calc_id):
     if remote:
         datadir = datastore.get_datadir()
         webex = WebExtractor(calc_id)
-        status = webex.status['status']
         hc_id = webex.oqparam.hazard_calculation_id
         if hc_id:
             sys.exit('The job has a parent (#%d) and cannot be '
@@ -54,8 +51,8 @@ def importcalc(calc_id):
         webex.dump('%s/calc_%d.hdf5' % (datadir, calc_id))
         webex.close()
     with datastore.read(calc_id) as dstore:
-        engine.expose_outputs(dstore, status=status)
+        engine.expose_outputs(dstore, status='complete')
     logging.info('Imported calculation %s successfully', calc_id)
 
 
-importcalc.arg('calc_id', 'calculation ID or pathname')
+main.calc_id = 'calculation ID or pathname'
