@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2020 GEM Foundation
+# Copyright (C) 2014-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -16,13 +16,13 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake. If not, see <http://www.gnu.org/licenses/>.
 import os
+import sys
 import unittest.mock as mock
 import logging
 import operator
 import collections
 import numpy
 from decorator import FunctionMaker
-from openquake.baselib import sap
 from openquake.baselib.general import groupby, gen_subclasses
 from openquake.baselib.performance import Monitor
 from openquake.hazardlib import gsim, nrml, imt
@@ -65,18 +65,6 @@ def source_model_info(sm_nodes):
     return rst_table(out)
 
 
-def print_full_lt(fname):
-    """
-    Parse the composite source model and
-    prints information about its composition and the full logic tree
-    """
-    oqparam = readinput.get_oqparam(fname)
-    full_lt = readinput.get_full_lt(oqparam)
-    print(full_lt)
-    print('See http://docs.openquake.org/oq-engine/stable/'
-          'effective-realizations.html for an explanation')
-
-
 def do_build_reports(directory):
     """
     Walk the directory and builds pre-calculation reports for all the
@@ -95,11 +83,10 @@ def do_build_reports(directory):
 
 
 choices = ['calculators', 'gsims', 'imts', 'views', 'exports',
-           'extracts', 'parameters', 'sources', 'mfds']
+           'extracts', 'parameters', 'sources', 'mfds', 'venv']
 
 
-@sap.script
-def info(what, report=False):
+def main(what, report=False):
     """
     Give information about the passed keyword or filename
     """
@@ -136,16 +123,20 @@ def info(what, report=False):
                 fm = FunctionMaker(func)
             print('%s(%s)%s' % (fm.name, fm.signature, fm.doc))
     elif what == 'parameters':
-        params = []
+        docs = OqParam.docs()
+        names = set()
         for val in vars(OqParam).values():
             if hasattr(val, 'name'):
-                params.append(val)
-        params.sort(key=lambda x: x.name)
+                names.add(val.name)
+        params = sorted(names)
         for param in params:
-            print(param.name)
+            print(param)
+            print(docs[param])
     elif what == 'mfds':
         for cls in gen_subclasses(BaseMFD):
             print(cls.__name__)
+    elif what == 'venv':
+        print(sys.prefix)
     elif what == 'sources':
         for cls in gen_subclasses(BaseSeismicSource):
             print(cls.__name__)
@@ -170,12 +161,12 @@ def info(what, report=False):
             if report:
                 print('Generated', reportwriter.build_report(what))
             else:
-                print_full_lt(what)
+                print(readinput.get_oqparam(what).json())
         if mon.duration > 1:
             print(mon)
     elif what:
         print("No info for '%s'" % what)
 
 
-info.arg('what', 'filename or one of %s' % ', '.join(choices))
-info.flg('report', 'build rst report from job.ini file or zip archive')
+main.what = 'filename or one of %s' % ', '.join(choices)
+main.report = 'build rst report from job.ini file or zip archive'
