@@ -17,13 +17,11 @@
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
 
 import logging
-from openquake.baselib import sap
 from openquake.commonlib.writers import write_csv
 from openquake.commonlib.logictree import SourceModelLogicTree
 
 
-@sap.script
-def ltcsv(fname, out=None):
+def main(fname, out=None):
     """
     Convert logic tree source model file from XML into CSV
     """
@@ -35,5 +33,5 @@ def ltcsv(fname, out=None):
     logging.info('Saved %s', out)
 
 
-ltcsv.arg('fname', 'path to the XML source model logic tree')
-ltcsv.arg('out', 'path to the CSV source model logic tree')
+main.fname = 'path to the XML source model logic tree'
+main.out = 'path to the CSV source model logic tree'
