@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2020 GEM Foundation
+# Copyright (C) 2015-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -16,18 +16,19 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake. If not, see <http://www.gnu.org/licenses/>.
 import gzip
+import json
 import logging
 import shapely
 import numpy
-from openquake.baselib import sap
+from openquake.hazardlib.geo.utils import cross_idl
 from openquake.hazardlib.contexts import Effect, get_effect_by_mag
-from openquake.hazardlib.calc.filters import getdefault, IntegrationDistance
+from openquake.hazardlib.calc.filters import getdefault, MagDepDistance
 from openquake.calculators.extract import Extractor, WebExtractor
 
 
 def make_figure_hcurves(extractors, what):
     """
-    $ oq plot 'hcurves?kind=mean&imt=PGA&site_id=0'
+    $ oq plot "hcurves?kind=mean&imt=PGA&site_id=0"
     """
     import matplotlib.pyplot as plt
     fig = plt.figure()
@@ -49,15 +50,52 @@ def make_figure_hcurves(extractors, what):
         for ck, arr in got.items():
             if (arr == 0).all():
                 logging.warning('There is a zero curve %s_%s', *ck)
-            ax.loglog(imls, arr[0], '-', label='%s_%s' % ck)
+            ax.loglog(imls, arr.flat, '-', label='%s_%s' % ck)
         ax.grid(True)
         ax.legend()
     return plt
 
 
+def make_figure_avg_gmf(extractors, what):
+    """
+    $ oq plot "avg_gmf?imt=gmv_0"
+    """
+    import matplotlib.pyplot as plt
+    fig = plt.figure()
+    [ex] = extractors
+    avg_gmf = ex.get(what)
+    imt = what.split('=')[1]
+    ax = fig.add_subplot(1, 1, 1)
+    ax.grid(True)
+    ax.set_title('Avg GMF for %s' % imt)
+    ax.set_xlabel('Lon')
+    ax.set_ylabel('Lat')
+    coll = ax.scatter(avg_gmf['lons'], avg_gmf['lats'],
+                      c=avg_gmf[imt], cmap='jet')
+    plt.colorbar(coll)
+    return plt
+
+
+def make_figure_vs30(extractors, what):
+    """
+    $ oq plot "vs30?"
+    """
+    import matplotlib.pyplot as plt
+    fig = plt.figure()
+    [ex] = extractors
+    sitecol = ex.get('sitecol')
+    ax = fig.add_subplot(111)
+    ax.grid(True)
+    ax.set_xlabel('vs30 for calculation %d' % ex.calc_id)
+    vs30 = sitecol['vs30']
+    vs30[numpy.isnan(vs30)] = 0
+    ax.scatter(sitecol['lon'], sitecol['lat'], c=vs30, cmap='jet')
+    return plt
+
+
 def make_figure_hmaps(extractors, what):
     """
-    $ oq plot 'hmaps?kind=mean&imt=PGA'
+    $ oq plot "hmaps?kind=mean&imt=PGA"
     """
     import matplotlib.pyplot as plt
     fig = plt.figure()
@@ -73,7 +111,9 @@ def make_figure_hmaps(extractors, what):
         itime = oq1.investigation_time
         assert oq2.investigation_time == itime
         sitecol = ex1.get('sitecol')
-        assert ex1.get('sitecol') == sitecol
+        array2 = ex2.get('sitecol').array
+        for name in ('lon', 'lat'):
+            numpy.testing.assert_equal(array2[name], sitecol.array[name])
         hmaps1 = ex1.get(what)
         hmaps2 = ex2.get(what)
         [imt] = hmaps1.imt
@@ -89,8 +129,9 @@ def make_figure_hmaps(extractors, what):
                           'inv_time=%dy\nmaxdiff=%s' %
                           (imt, kind, poe, ex1.calc_id, ex2.calc_id,
                            itime, maxdiff))
-            ax.scatter(sitecol['lon'], sitecol['lat'],
-                       c=diff, cmap='jet')
+            coll = ax.scatter(sitecol['lon'], sitecol['lat'],
+                              c=diff, cmap='jet')
+            plt.colorbar(coll)
     elif ncalcs == 1:  # plot the hmap
         [ex] = extractors
         oq = ex.oqparam
@@ -105,14 +146,15 @@ def make_figure_hmaps(extractors, what):
             ax.set_xlabel('hmap for IMT=%s, kind=%s, poe=%s\ncalculation %d, '
                           'inv_time=%dy' %
                           (imt, kind, poe, ex.calc_id, oq.investigation_time))
-            ax.scatter(sitecol['lon'], sitecol['lat'],
-                       c=hmaps[kind][:, 0, j], cmap='jet')
+            coll = ax.scatter(sitecol['lon'], sitecol['lat'],
+                              c=hmaps[kind][:, 0, j], cmap='jet')
+            plt.colorbar(coll)
     return plt
 
 
 def make_figure_uhs(extractors, what):
     """
-    $ oq plot 'uhs?kind=mean&site_id=0'
+    $ oq plot "uhs?kind=mean&site_id=0"
     """
     import matplotlib.pyplot as plt
     fig = plt.figure()
@@ -131,38 +173,90 @@ def make_figure_uhs(extractors, what):
                       (site, poe, oq.investigation_time))
         ax.set_ylabel('SA')
         for ck, arr in got.items():
-            ax.plot(periods, arr[0, :, j], '-', label='%s_%s' % ck)
-            ax.plot(periods, arr[0, :, j], '.')
+            curve = list(arr[site][str(poe)])
+            ax.plot(periods, curve, '-', label='%s_%s' % ck)
+            ax.plot(periods, curve, '.')
         ax.grid(True)
         ax.legend()
     return plt
 
 
+def middle(x):
+    # [1, 2, 3] => [1.5, 2.5]
+    return (x[:-1] + x[1:]) / 2.
+
+
+def stacked_bar(ax, x, ys, width):
+    cumsum = ys.cumsum(axis=0)
+    for i, y in enumerate(ys):
+        if i > 0:
+            ax.bar(x, y, width, bottom=cumsum[i-1])
+        else:
+            ax.bar(x, y, width)
+
+
 def make_figure_disagg(extractors, what):
     """
-    $ oq plot 'disagg?kind=Mag&imt=PGA&rlz=0'
+    $ oq plot "disagg?kind=Mag&imt=PGA"
     """
     import matplotlib.pyplot as plt
+    from matplotlib import cm
     fig = plt.figure()
-    ax = fig.add_subplot(1, 1, 1)
     oq = extractors[0].oqparam
     disagg = extractors[0].get(what)
     [sid] = disagg.site_id
     [imt] = disagg.imt
     [poe_id] = disagg.poe_id
-    [rlz] = disagg.rlz
-    ax.set_xlabel('Disagg%s on site %s, imt=%s, poe_id=%d, inv_time=%dy' %
-                  (disagg.kind, sid, imt, poe_id, oq.investigation_time))
-    x, y = disagg.array.T
+    y = disagg.array
     print(y)
-    ax.plot(x, y, label='rlz-%s' % rlz)
-    ax.legend()
+    ndims = len(y.shape)
+    axis = disagg.kind.split('_')
+    bins = getattr(disagg, axis[0])
+    ncalcs = len(extractors)
+    width = (bins[1] - bins[0]) * 0.5
+    x = middle(bins) if ncalcs == 1 else middle(bins) - width
+    if ndims == 1:  # simple bar chart
+        ax = fig.add_subplot(1, 1, 1)
+        ax.set_xlabel('Disagg%s on site %s, imt=%s, poe_id=%d, inv_time=%dy' %
+                      (disagg.kind, sid, imt, poe_id, oq.investigation_time))
+        ax.set_xlabel(axis[0])
+        ax.set_xticks(bins)
+        ax.bar(x, y, width)
+        for ex in extractors[1:]:
+            ax.bar(x + width, ex.get(what).array, width)
+        return plt
+    if ncalcs > 1:
+        raise NotImplementedError('Comparison for %s' % disagg.kind)
+    # 2D images
+    if ndims == 2:
+        y = y.reshape(y.shape + (1,))
+        zbins = ['']
+    else:
+        zbins = getattr(disagg, axis[2])
+    Z = y.shape[-1]
+    axes = []
+    for z in range(Z):
+        arr = y[:, :, z]
+        ax = fig.add_subplot(Z, 1, z + 1)
+        axes.append(ax)
+        ax.set_ylabel(axis[1])
+        ax.set_title(zbins[z])
+        vbins = getattr(disagg, axis[1])  # vertical bins
+        cmap = cm.get_cmap('jet', 100)
+        extent = bins[0], bins[-1], vbins[0], vbins[-1]
+        im = ax.imshow(arr, cmap=cmap, extent=extent,
+                       aspect='auto', vmin=y.min(), vmax=y.max())
+        # stacked bar chart
+        # stacked_bar(ax, x, y.T, width)
+        # ys = ['%.1f' % y for y in getattr(disagg, axis[1])]
+        # ax.legend(ys)
+    fig.colorbar(im, ax=axes)
     return plt
 
 
 def make_figure_task_info(extractors, what):
     """
-    $ oq plot 'task_info?kind=classical'
+    $ oq plot "task_info?kind=classical"
     """
     import matplotlib.pyplot as plt
     fig = plt.figure()
@@ -188,7 +282,7 @@ def make_figure_task_info(extractors, what):
 
 def make_figure_memory(extractors, what):
     """
-    $ oq plot 'memory?'
+    $ oq plot "memory?"
     """
     # NB: matplotlib is imported inside since it is a costly import
     import matplotlib.pyplot as plt
@@ -231,12 +325,13 @@ class PolygonPlotter():
         except ValueError:  # LINESTRING, not POLYGON
             pass
 
-    def set_lim(self, sitecol=None):
-        if sitecol:
-            self.minxs.append(min(sitecol['lon']))
-            self.maxxs.append(max(sitecol['lon']))
-            self.minys.append(min(sitecol['lat']))
-            self.maxys.append(max(sitecol['lat']))
+    def set_lim(self, xs=(), ys=()):
+        if len(xs):
+            self.minxs.append(min(xs))
+            self.maxxs.append(max(xs))
+        if len(ys):
+            self.minys.append(min(ys))
+            self.maxys.append(max(ys))
         if self.minxs and self.maxxs:
             self.ax.set_xlim(min(self.minxs), max(self.maxxs))
         if self.minys and self.maxys:
@@ -245,9 +340,9 @@ class PolygonPlotter():
 
 def make_figure_sources(extractors, what):
     """
-    $ oq plot sources?limit=100
-    $ oq plot sources?source_id=1&source_id=2
-    $ oq plot sources?code=A&code=N
+    $ oq plot "sources?limit=100"
+    $ oq plot "sources?source_id=1&source_id=2"
+    $ oq plot "sources?code=A&code=N"
     """
     # NB: matplotlib is imported inside since it is a costly import
     import matplotlib.pyplot as plt
@@ -258,10 +353,10 @@ def make_figure_sources(extractors, what):
     fig, ax = plt.subplots()
     ax.grid(True)
     sitecol = ex.get('sitecol')
-    ax.plot(sitecol['lon'], sitecol['lat'], '+')
     pp = PolygonPlotter(ax)
     n = 0
     tot = 0
+    psources = []
     for rec, srcid, wkt in zip(info, srcs, wkts):
         if not wkt:
             logging.warning('No geometries for source id %s', srcid)
@@ -273,16 +368,56 @@ def make_figure_sources(extractors, what):
         else:
             color = 'yellow'
             alpha = .1
-        pp.add(shapely.wkt.loads(wkt), alpha=alpha, color=color)
+        if wkt.startswith('POINT'):
+            psources.append(shapely.wkt.loads(wkt))
+        else:
+            pp.add(shapely.wkt.loads(wkt), alpha=alpha, color=color)
         tot += 1
-    pp.set_lim(sitecol)
+    lons = [p.x for p in psources]
+    lats = [p.y for p in psources]
+    ss_lons = lons + list(sitecol['lon'])  # sites + sources longitudes
+    ss_lats = lats + list(sitecol['lat'])  # sites + sources latitudes
+    if len(ss_lons) > 1 and cross_idl(*ss_lons):
+        ss_lons = [lon % 360 for lon in ss_lons]
+        lons = [lon % 360 for lon in lons]
+        sitecol['lon'] = sitecol['lon'] % 360
+    ax.plot(sitecol['lon'], sitecol['lat'], '.')
+    ax.plot(lons, lats, 'o')
+    pp.set_lim(ss_lons, ss_lats)
     ax.set_title('%d/%d sources' % (n, tot))
     return plt
 
 
+def make_figure_gridded_sources(extractors, what):
+    """
+    $ oq plot "gridded_sources?task_no=0"
+    """
+    # NB: matplotlib is imported inside since it is a costly import
+    import matplotlib.pyplot as plt
+    [ex] = extractors
+    dic = json.loads(ex.get(what).json)  # id -> lonlats
+    fig, ax = plt.subplots()
+    ax.grid(True)
+    sitecol = ex.get('sitecol')
+    tot = 0
+    for lonlats in dic.values():
+        if len(lonlats) == 2:  # not collapsed
+            tot += 1
+        else:  # collapsed
+            tot += len(lonlats) / 2 - 1
+        ax.plot([lonlats[0]], [lonlats[1]], '*')
+        lons = lonlats[2::2]
+        lats = lonlats[3::2]
+        ax.scatter(lons, lats)
+    ax.plot(sitecol['lon'], sitecol['lat'], '.')
+    ax.set_title('Reduced %d->%d sources' % (tot, len(dic)))
+    # TODO: fix plot around the IDL
+    return plt
+
+
 def make_figure_rupture_info(extractors, what):
     """
-    $ oq plot rupture_info?min_mag=6
+    $ oq plot "rupture_info?min_mag=6"
     """
     # NB: matplotlib is imported inside since it is a costly import
     import matplotlib.pyplot as plt
@@ -312,7 +447,7 @@ def make_figure_rupture_info(extractors, what):
 
 def make_figure_effect(extractors, what):
     """
-    $ oq plot 'effect?'
+    $ oq plot "effect?"
     """
     # NB: matplotlib is imported inside since it is a costly import
     import matplotlib.pyplot as plt
@@ -343,7 +478,7 @@ def make_figure_effect(extractors, what):
 
 def make_figure_rups_by_mag_dist(extractors, what):
     """
-    $ oq plot 'rups_by_mag_dist?'
+    $ oq plot "rups_by_mag_dist?"
     """
     # NB: matplotlib is imported inside since it is a costly import
     import matplotlib.pyplot as plt
@@ -374,7 +509,7 @@ def make_figure_rups_by_mag_dist(extractors, what):
 
 def make_figure_dist_by_mag(extractors, what):
     """
-    $ oq plot 'dist_by_mag?'
+    $ oq plot "dist_by_mag?"
     """
     # NB: matplotlib is imported inside since it is a costly import
     import matplotlib.pyplot as plt
@@ -407,7 +542,7 @@ def make_figure_dist_by_mag(extractors, what):
 
 def make_figure_effect_by_mag(extractors, what):
     """
-    $ oq plot 'effect_by_mag?'
+    $ oq plot "effect_by_mag?"
     """
     # NB: matplotlib is imported inside since it is a costly import
     import matplotlib.pyplot as plt
@@ -418,7 +553,7 @@ def make_figure_effect_by_mag(extractors, what):
         effect = ex.get('effect')
     except KeyError:
         onesite = ex.get('sitecol').one()
-        maximum_distance = IntegrationDistance(ex.oqparam.maximum_distance)
+        maximum_distance = MagDepDistance(ex.oqparam.maximum_distance)
         imtls = ex.oqparam.imtls
         ebm = get_effect_by_mag(
             mags, onesite, gsims_by_trt, maximum_distance, imtls)
@@ -437,7 +572,7 @@ def make_figure_effect_by_mag(extractors, what):
 
 def make_figure_agg_curves(extractors, what):
     """
-    $ oq plot 'agg_curves?kind=mean&loss_type=structural' -1
+    $ oq plot "agg_curves?kind=mean&loss_type=structural" -1
     """
     import matplotlib.pyplot as plt
     fig = plt.figure()
@@ -461,29 +596,45 @@ def make_figure_agg_curves(extractors, what):
 
 def make_figure_tot_curves(extractors, what):
     """
-    $ oq plot 'tot_curves?loss_type=structural&kind=rlz-000&absolute=1'
+    $ oq plot "tot_curves?loss_type=structural&kind=rlz-000&absolute=1"
     """
     import matplotlib.pyplot as plt
     fig = plt.figure()
     [ex] = extractors
     tot = ex.get(what)
-    app = ex.get(what.replace('tot_', 'app_'))
     ax = fig.add_subplot(1, 1, 1)
     ax.set_xlabel('return periods')
     ax.set_ylabel('PoE')
     ax.loglog(tot.return_period, tot[:, 0], '-', label='tot_curves')
-    ax.loglog(app.return_period, app[:, 0], '-', label='app_curves')
     ax.grid(True)
     ax.legend()
     return plt
 
 
-@sap.script
-def plot(what='examples', calc_id=-1, other_id=None, webapi=False,
+def plot_wkt(wkt_string):
+    """
+    Plot a WKT string describing a polygon
+    """
+    from shapely import wkt
+    import matplotlib.pyplot as plt
+    poly = wkt.loads(wkt_string)
+    coo = numpy.array(poly.exterior.coords)
+    plt.plot(coo[:, 0], coo[:, 1], '-')
+    return plt
+
+
+def main(what,
+         calc_id: int = -1,
+         others: int = [],
+         webapi=False,
          local=False):
     """
     Generic plotter for local and remote calculations.
     """
+    if what.startswith('POLYGON'):
+        plt = plot_wkt(what)
+        plt.show()
+        return
     if what == 'examples':
         help_msg = ['Examples of possible plots:']
         for k, v in globals().items():
@@ -503,23 +654,23 @@ def plot(what='examples', calc_id=-1, other_id=None, webapi=False,
         what += '&poe_id=0'
     if local:
         xs = [WebExtractor(calc_id, 'http://localhost:8800', '')]
-        if other_id:
+        for other_id in others:
             xs.append(WebExtractor(other_id), 'http://localhost:8800', '')
     elif webapi:
         xs = [WebExtractor(calc_id)]
-        if other_id:
+        for other_id in others:
             xs.append(WebExtractor(other_id))
     else:
         xs = [Extractor(calc_id)]
-        if other_id:
+        for other_id in others:
             xs.append(Extractor(other_id))
     make_figure = globals()['make_figure_' + prefix]
     plt = make_figure(xs, what)
     plt.show()
 
 
-plot.arg('what', 'what to extract')
-plot.arg('calc_id', 'computation ID', type=int)
-plot.arg('other_id', 'ID of another computation', type=int)
-plot.flg('webapi', 'if given, pass through the WebAPI')
-plot.flg('local', 'if passed, use the local WebAPI')
+main.what = 'what to extract (try examples)'
+main.calc_id = 'computation ID'
+main.others = dict(help='IDs of other computations', nargs='*')
+main.webapi = 'if given, pass through the WebAPI'
+main.local = 'if passed, use the local WebAPI'
