@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2020 GEM Foundation
+# Copyright (C) 2018-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -15,17 +15,17 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
+import os
 import logging
 import numpy
-from openquake.baselib import sap, performance, datastore
+from openquake.baselib import performance, datastore
 from openquake.hazardlib import site, valid
 from openquake.hazardlib.geo.utils import assoc
 from openquake.risklib.asset import Exposure
 from openquake.commonlib.writers import write_csv
 
-F32 = numpy.float32
 SQRT2 = 1.414
-vs30_dt = numpy.dtype([('lon', F32), ('lat', F32), ('vs30', F32)])
+vs30_dt = numpy.dtype([('lon', float), ('lat', float), ('vs30', float)])
 
 
 # TODO: equivalents of calculate_z1pt0 and calculate_z2pt5_ngaw2
@@ -59,22 +59,40 @@ def calculate_z2pt5_ngaw2(vs30):
     return z2pt5
 
 
-def read_vs30(fnames):
+def read_vs30(fnames, forbidden):
     """
     :param fnames: a list of CSV files with fields lon,lat,vs30
+    :param forbidden: forbidden name for the input files
     :returns: a vs30 array of dtype vs30dt
     """
     data = []
     for fname in fnames:
+        check_fname(fname, 'vs30_csv', forbidden)
         for line in open(fname, encoding='utf-8-sig'):
             data.append(tuple(line.split(',')))
     return numpy.array(data, vs30_dt)
 
 
-@sap.script
-def prepare_site_model(exposure_xml, sites_csv, vs30_csv,
-                       z1pt0, z2pt5, vs30measured, grid_spacing=0,
-                       assoc_distance=5, output='site_model.csv'):
+def check_fname(fname, kind, forbidden):
+    """
+    Raise a NameError if fname == forbidden
+    """
+    if os.path.basename(fname).lower() == forbidden:
+        raise NameError('A file of kind %s cannot be called %r!'
+                        % (kind, forbidden))
+
+
+def main(
+        vs30_csv,
+        z1pt0=False,
+        z2pt5=False,
+        vs30measured=False,
+        *,
+        exposure_xml=None,
+        sites_csv=None,
+        grid_spacing: float = 0,
+        assoc_distance: float = 5,
+        output='site_model.csv'):
     """
     Prepare a site_model.csv file from exposure xml files/site csv files,
     vs30 csv files and a grid spacing which can be 0 (meaning no grid).
@@ -122,6 +140,7 @@ def prepare_site_model(exposure_xml, sites_csv, vs30_csv,
         elif sites_csv:
             lons, lats = [], []
             for fname in sites_csv:
+                check_fname(fname, 'sites_csv', output)
                 with open(fname) as csv:
                     for line in csv:
                         if line.startswith('lon,lat'):  # possible header
@@ -132,13 +151,13 @@ def prepare_site_model(exposure_xml, sites_csv, vs30_csv,
             haz_sitecol = site.SiteCollection.from_points(
                 lons, lats, req_site_params=req_site_params)
             if grid_spacing:
-                grid = mesh.get_convex_hull().dilate(
+                grid = haz_sitecol.mesh.get_convex_hull().dilate(
                     grid_spacing).discretize(grid_spacing)
                 haz_sitecol = site.SiteCollection.from_points(
                     grid.lons, grid.lats, req_site_params=req_site_params)
         else:
             raise RuntimeError('Missing exposures or missing sites')
-        vs30orig = read_vs30(vs30_csv)
+        vs30orig = read_vs30(vs30_csv, output)
         logging.info('Associating %d hazard sites to %d site parameters',
                      len(haz_sitecol), len(vs30orig))
         vs30 = haz_sitecol.assoc(vs30orig, assoc_distance,
@@ -154,15 +173,12 @@ def prepare_site_model(exposure_xml, sites_csv, vs30_csv,
     return haz_sitecol
 
 
-prepare_site_model.opt('exposure_xml', 'exposure(s) in XML format', nargs='*')
-prepare_site_model.opt('sites_csv', 'sites in CSV format', nargs='*')
-prepare_site_model.arg('vs30_csv', 'files with lon,lat,vs30 and no header',
-                       nargs='+')
-prepare_site_model.flg('z1pt0', 'build the z1pt0', '-1')
-prepare_site_model.flg('z2pt5', 'build the z2pt5', '-2')
-prepare_site_model.flg('vs30measured', 'build the vs30measured', '-3')
-prepare_site_model.opt('grid_spacing', 'grid spacing in km '
-                       '(the default 0 means no grid)', type=float)
-prepare_site_model.opt('assoc_distance',
-                       'sites over this distance are discarded', type=float)
-prepare_site_model.opt('output', 'output file')
+main.vs30_csv = dict(help='files with lon,lat,vs30 and no header', nargs='+')
+main.z1pt0 = dict(help='build the z1pt0', abbrev='-1')
+main.z2pt5 = dict(help='build the z2pt5', abbrev='-2')
+main.vs30measured = dict(help='build the vs30measured', abbrev='-3')
+main.exposure_xml = dict(help='exposure(s) in XML format', nargs='*')
+main.sites_csv = dict(help='sites in CSV format', nargs='*')
+main.grid_spacing = 'grid spacing in km (the default 0 means no grid)'
+main.assoc_distance = 'sites over this distance are discarded'
+main.output = 'output file'
