@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2020 GEM Foundation
+# Copyright (C) 2014-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -16,12 +16,10 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
 import os.path
-from openquake.baselib import sap
 from openquake.commonlib import shapefileparser
 
 
-@sap.script
-def to_shapefile(output, input_nrml_file, validate):
+def main(input_nrml_file, validate=False, *, output=None):
     """
     Convert a NRML source model file to ESRI Shapefile(s).
 
@@ -40,6 +38,6 @@ def to_shapefile(output, input_nrml_file, validate):
     shapefileparser.ShapefileParser().write(output, source_model)
 
 
-to_shapefile.opt('output', 'output path (no extension)')
-to_shapefile.arg('input_nrml_file', 'path to source model NRML file')
-to_shapefile.flg('validate', 'Apply validation to input model (can be slow)')
+main.input_nrml_file = 'path to source model NRML file'
+main.validate = 'Apply validation to input model (can be slow)'
+main.output = 'output path (no extension)'
