@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2017-2020 GEM Foundation
+# Copyright (C) 2017-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -17,12 +17,10 @@
 # along with OpenQuake. If not, see <http://www.gnu.org/licenses/>.
 import sys
 import os.path
-from openquake.baselib import sap
 from openquake.commonlib import oqzip
 
 
-@sap.script
-def zip(what, archive_zip='', risk_file=''):
+def main(what, archive_zip='', *, risk_file=''):
     """
     Zip into an archive one or two job.ini files with all related files
     """
@@ -40,7 +38,7 @@ def zip(what, archive_zip='', risk_file=''):
         sys.exit('Cannot zip %s' % what)
 
 
-zip.arg('what', 'path to a job.ini, an ssmLT.xml, an exposure.xml, '
-        'or to a directory containing a file ssmLT.xml')
-zip.arg('archive_zip', 'path to a non-existing .zip file')
-zip.opt('risk_file', 'optional file for risk')
+main.what = ('path to a job.ini, an ssmLT.xml, an exposure.xml, '
+             'or to a directory containing a file ssmLT.xml')
+main.archive_zip = 'path to a non-existing .zip file'
+main.risk_file = 'optional file for risk'
