@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2010-2020 GEM Foundation
+# Copyright (C) 2010-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -28,6 +28,7 @@ import io
 import os
 import re
 import time
+import string
 import logging
 import functools
 import itertools
@@ -39,15 +40,14 @@ import numpy
 from openquake.baselib import hdf5
 from openquake.baselib.python3compat import decode
 from openquake.baselib.node import node_from_elem, Node as N, context
-from openquake.baselib.general import (groupby, group_array, duplicated,
-                                       add_defaults, AccumDict)
+from openquake.baselib.general import groupby, duplicated, AccumDict
 from openquake.hazardlib.gsim.mgmpe.avg_gmpe import AvgGMPE
 from openquake.hazardlib.gsim.base import CoeffsTable
 from openquake.hazardlib.imt import from_string
 from openquake.hazardlib import valid, nrml, InvalidFile, pmf
 from openquake.hazardlib.sourceconverter import SourceGroup
 from openquake.hazardlib.lt import (
-    Branch, BranchSet, LogicTreeError, parse_uncertainty, sample)
+    Branch, BranchSet, LogicTreeError, parse_uncertainty, sample, random)
 
 TRT_REGEX = re.compile(r'tectonicRegion="([^"]+?)"')
 ID_REGEX = re.compile(r'id="([^"]+?)"')
@@ -69,11 +69,10 @@ source_model_dt = numpy.dtype([
     ('weight', F32),
     ('path', hdf5.vstr),
     ('samples', U32),
-    ('offset', U32),
 ])
 
 src_group_dt = numpy.dtype(
-    [('grp_id', U32),
+    [('et_id', U32),
      ('name', hdf5.vstr),
      ('trti', U16),
      ('effrup', I32),
@@ -101,19 +100,18 @@ def unique(objects, key=None):
 class Realization(object):
     """
     Generic Realization object with attributes value, weight, ordinal, lt_path,
-    samples and optionally offset.
+    samples.
     """
-    def __init__(self, value, weight, ordinal, lt_path, samples, offset=0):
+    def __init__(self, value, weight, ordinal, lt_path, samples=1):
         self.value = value
         self.weight = weight
         self.ordinal = ordinal
         self.lt_path = lt_path
         self.samples = samples
-        self.offset = offset
 
     @property
     def pid(self):
-        return '_'.join(self.lt_path)  # path ID
+        return '~'.join(self.lt_path)  # path ID
 
     @property
     def name(self):
@@ -132,7 +130,7 @@ class Realization(object):
         samples = ', samples=%d' % self.samples if self.samples > 1 else ''
         return '<%s #%d %s, path=%s, weight=%s%s>' % (
             self.__class__.__name__, self.ordinal, self.value,
-            '_'.join(self.lt_path), self.weight, samples)
+            '~'.join(self.lt_path), self.weight, samples)
 
 
 @functools.lru_cache()
@@ -175,7 +173,8 @@ Info = collections.namedtuple('Info', 'smpaths, applytosources')
 def collect_info(smlt):
     """
     Given a path to a source model logic tree, collect all of the
-    path names to the source models it contains and build
+    path names to the source models it contains and build:
+
     1. a dictionary source model branch ID -> paths
     2. a dictionary source model branch ID -> source IDs in applyToSources
 
@@ -233,6 +232,32 @@ def read_source_groups(fname):
     return src_groups
 
 
+def keyno(branch_id, no, fname='',
+          chars=string.digits + string.ascii_uppercase):
+    """
+    :param branch_id: a branch ID string
+    :param no: number of the branch in the branchset (starting from 0)
+    :returns: a 1-char string for the branch_id based on the branch number
+    """
+    try:
+        valid.branch_id(branch_id)
+    except ValueError as ex:
+        raise ValueError('%s %s' % (ex, fname))
+    try:
+        return chars[no]
+    except IndexError:
+        return branch_id
+
+
+def shorten(path, shortener):
+    """
+    :path:  sequence of strings
+    :shortener: dictionary longstring -> shortstring
+    :returns: shortened version of the path
+    """
+    return ''.join(shortener.get(key, key) for key in path)
+
+
 class SourceModelLogicTree(object):
     """
     Source model logic tree parser.
@@ -257,20 +282,25 @@ class SourceModelLogicTree(object):
         self = object.__new__(cls)
         arr = numpy.array([('bs0', 'b0', 'sourceModel', 'fake.xml', 1)],
                           branch_dt)
-        dic = dict(filename='fake.xml', seed=0, num_samples=0)
+        dic = dict(filename='fake.xml', seed=0, num_samples=0,
+                   sampling_method='early_weights')
         self.__fromh5__(arr, dic)
         return self
 
-    def __init__(self, filename, seed=0, num_samples=0):
+    def __init__(self, filename, seed=0, num_samples=0,
+                 sampling_method='early_weights'):
         self.filename = filename
         self.basepath = os.path.dirname(filename)
-        self.seed = seed
+        # NB: converting the random_seed into an integer is needed on Windows
+        self.seed = int(seed)
         self.num_samples = num_samples
+        self.sampling_method = sampling_method
         self.branches = {}  # branch_id -> branch
         self.bsetdict = {}
         self.previous_branches = []
         self.tectonic_region_types = set()
         self.source_types = set()
+        self.hdf5_files = set()
         self.root_branchset = None
         root = nrml.read(filename)
         try:
@@ -278,6 +308,7 @@ class SourceModelLogicTree(object):
         except AttributeError:
             raise LogicTreeError(
                 root, self.filename, "missing logicTree node")
+        self.shortener = {}
         self.parse_tree(tree)
 
     @property
@@ -320,7 +351,6 @@ class SourceModelLogicTree(object):
         can have child branchsets (if there is one on the next level).
         """
         attrs = branchset_node.attrib.copy()
-        self.bsetdict[attrs.pop('branchSetID')] = attrs
         uncertainty_type = branchset_node.attrib.get('uncertaintyType')
         filters = dict((filtername, branchset_node.attrib.get(filtername))
                        for filtername in self.FILTERS
@@ -328,7 +358,8 @@ class SourceModelLogicTree(object):
         self.validate_filters(branchset_node, uncertainty_type, filters)
 
         filters = self.parse_filters(branchset_node, uncertainty_type, filters)
-        branchset = BranchSet(uncertainty_type, filters)
+        branchset = BranchSet(uncertainty_type, len(self.bsetdict), filters)
+        self.bsetdict[attrs.pop('branchSetID')] = attrs
         self.validate_branchset(branchset_node, depth, branchset)
 
         self.parse_branches(branchset_node, branchset)
@@ -365,7 +396,7 @@ class SourceModelLogicTree(object):
         weight_sum = 0
         branches = branchset_node.nodes
         values = []
-        for branchnode in branches:
+        for no, branchnode in enumerate(branches):
             weight = ~branchnode.uncertaintyWeight
             weight_sum += weight
             value_node = node_from_elem(branchnode.uncertaintyModel)
@@ -389,6 +420,7 @@ class SourceModelLogicTree(object):
                     branchnode, self.filename,
                     "branchID '%s' is not unique" % branch_id)
             self.branches[branch_id] = branch
+            self.shortener[branch_id] = keyno(branch_id, no, self.filename)
             branchset.branches.append(branch)
         if abs(weight_sum - 1.0) > pmf.PRECISION:
             raise LogicTreeError(
@@ -408,19 +440,27 @@ class SourceModelLogicTree(object):
         """
         if self.num_samples:
             # random sampling of the logic tree
-            weight = 1. / self.num_samples
-            for i in range(self.num_samples):
-                smlt_path = self.root_branchset.sample(self.seed + i)
-                name = smlt_path[0].value
-                smlt_path_ids = [branch.branch_id for branch in smlt_path]
-                yield Realization(name, weight, None, tuple(smlt_path_ids), 1)
+            probs = random((self.num_samples, len(self.bsetdict)),
+                           self.seed, self.sampling_method)
+            ordinal = 0
+            for branches in self.root_branchset.sample(
+                    probs, self.sampling_method):
+                name = branches[0].value
+                smlt_path_ids = [br.branch_id for br in branches]
+                if self.sampling_method.startswith('early_'):
+                    weight = 1. / self.num_samples  # already accounted
+                elif self.sampling_method.startswith('late_'):
+                    weight = numpy.prod([br.weight for br in branches])
+                else:
+                    raise NotImplementedError(self.sampling_method)
+                yield Realization(name, weight, ordinal, tuple(smlt_path_ids))
+                ordinal += 1
         else:  # full enumeration
             ordinal = 0
-            for weight, smlt_path in self.root_branchset.enumerate_paths():
-                name = smlt_path[0].value
-                smlt_branch_ids = [branch.branch_id for branch in smlt_path]
-                yield Realization(name, weight, ordinal,
-                                  tuple(smlt_branch_ids), 1)
+            for weight, branches in self.root_branchset.enumerate_paths():
+                name = branches[0].value  # source model name
+                branch_ids = [branch.branch_id for branch in branches]
+                yield Realization(name, weight, ordinal, tuple(branch_ids))
                 ordinal += 1
 
     def parse_filters(self, branchset_node, uncertainty_type, filters):
@@ -574,9 +614,12 @@ class SourceModelLogicTree(object):
         information is used then for :meth:`validate_filters` and
         :meth:`validate_uncertainty_value`.
         """
-        # using regular expressions is a lot faster than using the
+        # using regular expressions is a lot faster than parsing
         with self._get_source_model(source_model) as sm:
             xml = sm.read()
+        hdf5_file = os.path.splitext(source_model)[0] + '.hdf5'
+        if os.path.exists(hdf5_file):
+            self.hdf5_files.add(hdf5_file)
         self.tectonic_region_types.update(TRT_REGEX.findall(xml))
         self.source_ids[branch_id].extend(ID_REGEX.findall(xml))
         self.source_types.update(SOURCE_TYPE_REGEX.findall(xml))
@@ -612,32 +655,45 @@ class SourceModelLogicTree(object):
         attrs = self._tomldict()
         attrs['seed'] = self.seed
         attrs['num_samples'] = self.num_samples
+        attrs['sampling_method'] = self.sampling_method
         attrs['filename'] = self.filename
         return numpy.array(tbl, branch_dt), attrs
 
     def __fromh5__(self, array, attrs):
+        # this is rather tricky; to understand it, run the test
+        # SerializeSmltTestCase which has a logic tree with 3 branchsets
+        # with the form b11[b21[b31, b32], b22[b31, b32]] and 1 x 2 x 2 rlzs
         bsets = []
         self.branches = {}
         self.bsetdict = {}
-        for bsid, rows in group_array(array, 'branchset').items():
+        self.shortener = {}
+        acc = AccumDict(accum=[])  # bsid -> rows
+        for rec in array:
+            # NB: it is important to keep the order of the branchsets
+            acc[rec['branchset']].append(rec)
+        for ordinal, (bsid, rows) in enumerate(acc.items()):
             utype = rows[0]['utype']
-            bset = BranchSet(utype, [])  # TODO: filters
+            bset = BranchSet(utype, ordinal, filters=[])  # TODO: filters
             bset.id = bsid
-            for row in rows:
+            for no, row in enumerate(rows):
                 br = Branch(bsid, row['branch'], row['weight'], row['uvalue'])
                 self.branches[br.branch_id] = br
+                self.shortener[br.branch_id] = keyno(
+                    br.branch_id, no, attrs['filename'])
                 bset.branches.append(br)
             bsets.append(bset)
             self.bsetdict[bsid] = {'uncertaintyType': utype}
+        # bsets [<b11>, <b21 b22>, <b31 b32>]
         self.root_branchset = bsets[0]
         for i, childset in enumerate(bsets[1:]):
             dic = toml.loads(attrs[childset.id])
             atb = dic.get('applyToBranches')
             for branch in bsets[i].branches:  # parent branches
                 if not atb or branch.branch_id in atb:
-                    branch.bset = bset
+                    branch.bset = childset
         self.seed = attrs['seed']
         self.num_samples = attrs['num_samples']
+        self.sampling_method = attrs['sampling_method']
         self.filename = attrs['filename']
 
     def __str__(self):
@@ -761,6 +817,7 @@ class GsimLogicTree(object):
         self.values = collections.defaultdict(list)  # {trt: gsims}
         self._ltnode = ltnode or nrml.read(fname).logicTree
         self.bs_id_by_trt = {}
+        self.shortener = {}
         self.branches = self._build_trts_branches(trts)  # sorted by trt
         if trts != ['*']:
             # reduce self.values to the listed TRTs
@@ -809,7 +866,7 @@ class GsimLogicTree(object):
         dt = [('trt', hdf5.vstr), ('branch', hdf5.vstr),
               ('uncertainty', hdf5.vstr)] + [
             (weight, float) for weight in sorted(weights)]
-        branches = [(b.trt, b.id, b.gsim) +
+        branches = [(b.trt, b.id, repr(b.gsim)) +
                     tuple(b.weight[weight] for weight in sorted(weights))
                     for b in self.branches if b.effective]
         dic = {}
@@ -827,8 +884,9 @@ class GsimLogicTree(object):
 
     def __fromh5__(self, array, dic):
         self.branches = []
+        self.shortener = {}
         self.values = collections.defaultdict(list)
-        for branch in array:
+        for no, branch in enumerate(array):
             br_id = branch['branch']
             gsim = valid.gsim(branch['uncertainty'])
             for k, v in gsim.kwargs.items():
@@ -844,6 +902,7 @@ class GsimLogicTree(object):
                 gsim.weight = weight
             bt = BranchTuple(branch['trt'], br_id, gsim, weight, True)
             self.branches.append(bt)
+            self.shortener[br_id] = keyno(br_id, no)
 
     def reduce(self, trts):
         """
@@ -890,6 +949,7 @@ class GsimLogicTree(object):
                 _toml = toml.dumps({'AvgGMPE': kwargs})
                 gsim = AvgGMPE(**kwargs)
                 gsim._toml = _toml
+                new.values[trt] = [gsim]
                 branch = BranchTuple(trt, bs_id, gsim, sum(weights), True)
                 new.branches.append(branch)
             else:
@@ -911,8 +971,6 @@ class GsimLogicTree(object):
         """
         Return the effective number of paths in the tree.
         """
-        # NB: the algorithm assume a symmetric logic tree for the GSIMs;
-        # in the future we may relax such assumption
         num_branches = self.get_num_branches()
         if not sum(num_branches.values()):
             return 0
@@ -951,7 +1009,7 @@ class GsimLogicTree(object):
                          trt in tectonic_region_types)
             weights = []
             branch_ids = []
-            for branch in branchset:
+            for no, branch in enumerate(branchset):
                 weight = ImtWeight(branch, self.filename)
                 weights.append(weight)
                 branch_id = branch['branchID']
@@ -972,6 +1030,8 @@ class GsimLogicTree(object):
                     branch_id, gsim, weight, effective)
                 if effective:
                     branches.append(bt)
+                    self.shortener[branch_id] = keyno(
+                        branch_id, no, self.filename)
             tot = sum(weights)
             assert tot.is_one(), '%s in branch %s' % (tot, branch_id)
             if duplicated(branch_ids):
@@ -995,14 +1055,18 @@ class GsimLogicTree(object):
             [trt] = self.values
         return sorted(self.values[trt])
 
-    def sample(self, n, seed):
+    def sample(self, n, seed, sampling_method):
         """
         :param n: number of samples
         :param seed: random seed
+        :param sampling_method: by default 'early_weights'
         :returns: n Realization objects
         """
+        m = len(self.values)  # number of TRTs
+        probs = random((n, m), seed, sampling_method)
         brlists = [sample([b for b in self.branches if b.trt == trt],
-                          n, seed + i) for i, trt in enumerate(self.values)]
+                          probs[:, i], sampling_method)
+                   for i, trt in enumerate(self.values)]
         rlzs = []
         for i in range(n):
             weight = 1
@@ -1015,7 +1079,7 @@ class GsimLogicTree(object):
                 lt_uid.append(branch.id if branch.effective else '@')
                 weight *= branch.weight
                 value.append(branch.gsim)
-            rlz = Realization(tuple(value), weight, i, tuple(lt_uid), 1)
+            rlz = Realization(tuple(value), weight, i, tuple(lt_uid))
             rlzs.append(rlz)
         return rlzs
 
@@ -1038,7 +1102,7 @@ class GsimLogicTree(object):
                 lt_uid.append(branch.id if branch.effective else '@')
                 weight *= branch.weight
                 value.append(branch.gsim)
-            yield Realization(tuple(value), weight, i, tuple(lt_uid), 1)
+            yield Realization(tuple(value), weight, i, tuple(lt_uid))
 
     def __repr__(self):
         lines = ['%s,%s,%s,w=%s' %
@@ -1047,34 +1111,6 @@ class GsimLogicTree(object):
         return '<%s\n%s>' % (self.__class__.__name__, '\n'.join(lines))
 
 
-def taxonomy_mapping(filename, taxonomies):
-    """
-    :param filename: path to the CSV file containing the taxonomy associations
-    :param taxonomies: an array taxonomy string -> taxonomy index
-    :returns: (array, [[(taxonomy, weight), ...], ...])
-    """
-    if filename is None:  # trivial mapping
-        return (), [[(taxo, 1)] for taxo in taxonomies]
-    dic = {}  # taxonomy index -> risk taxonomy
-    array = hdf5.read_csv(filename, {None: hdf5.vstr, 'weight': float}).array
-    arr = add_defaults(array, weight=1.)
-    assert arr.dtype.names == ('taxonomy', 'conversion', 'weight')
-    dic = group_array(arr, 'taxonomy')
-    taxonomies = taxonomies[1:]  # strip '?'
-    missing = set(taxonomies) - set(dic)
-    if missing:
-        raise InvalidFile('The taxonomies %s are in the exposure but not in %s'
-                          % (missing, filename))
-    lst = [[("?", 1)]]
-    for idx, taxo in enumerate(taxonomies, 1):
-        recs = dic[taxo]
-        if abs(recs['weight'].sum() - 1.) > pmf.PRECISION:
-            raise InvalidFile('%s: the weights do not sum up to 1 for %s' %
-                              (filename, taxo))
-        lst.append([(rec['conversion'], rec['weight']) for rec in recs])
-    return arr, lst
-
-
 def capitalize(words):
     """
     Capitalize words separated by spaces.
@@ -1104,17 +1140,12 @@ class LtRealization(object):
         self.weight = weight
 
     def __repr__(self):
-        return '<%d,%s,w=%s>' % (self.ordinal, self.pid, self.weight)
+        return '<%d,w=%s>' % (self.ordinal, self.weight)
 
     @property
     def gsim_lt_path(self):
         return self.gsim_rlz.lt_path
 
-    @property
-    def pid(self):
-        """An unique identifier for effective realizations"""
-        return '_'.join(self.sm_lt_path) + '~' + self.gsim_rlz.pid
-
     def __lt__(self, other):
         return self.ordinal < other.ordinal
 
@@ -1145,45 +1176,45 @@ class FullLogicTree(object):
         gsim_lt = gsimlt or GsimLogicTree.from_('[FromFile]')
         fakeSM = Realization(
             'scenario', weight=1,  ordinal=0, lt_path='b1', samples=1)
-        info = object.__new__(cls)
-        info.source_model_lt = SourceModelLogicTree.fake()
-        info.gsim_lt = gsim_lt
-        info.sm_rlzs = [fakeSM]
-        return info
+        self = object.__new__(cls)
+        self.source_model_lt = SourceModelLogicTree.fake()
+        self.gsim_lt = gsim_lt
+        self.sm_rlzs = [fakeSM]
+        return self
 
     def __init__(self, source_model_lt, gsim_lt):
         self.source_model_lt = source_model_lt
         self.gsim_lt = gsim_lt
-        self.init()  # set .sm_rlzs and .trt_by_grp
+        self.init()  # set .sm_rlzs and .trt_by_et
 
     def init(self):
-        # NB: the number of effective rlzs can be less than the number
-        # of realizations in case of sampling
-        sm_rlzs = get_effective_rlzs(self.source_model_lt)
-        if not self.num_samples:
-            num_gsim_rlzs = self.gsim_lt.get_num_paths()
-        offset = 0
-        for sm_rlz in sm_rlzs:
-            sm_rlz.offset = offset
-            if self.num_samples:
-                offset += sm_rlz.samples
-            else:
-                offset += num_gsim_rlzs
-        self.sm_rlzs = sm_rlzs
+        if self.source_model_lt.num_samples:
+            # NB: the number of effective rlzs can be less than the number
+            # of realizations in case of sampling
+            self.sm_rlzs = get_effective_rlzs(self.source_model_lt)
+        else:  # full enumeration
+            samples = self.gsim_lt.get_num_paths()
+            self.sm_rlzs = []
+            for sm_rlz in self.source_model_lt:
+                sm_rlz.samples = samples
+                self.sm_rlzs.append(sm_rlz)
         self.trti = {trt: i for i, trt in enumerate(self.gsim_lt.values)}
 
+    def get_eri_by_ltp(self):
+        """
+        :returns: a dictionary sm_lt_path -> effective realization index
+        """
+        return {'~'.join(sm_rlz.lt_path): i
+                for i, sm_rlz in enumerate(self.sm_rlzs)}
+
     @property
-    def trt_by_grp(self):
+    def trt_by_et(self):
         """
-        :returns: a dictionary grp_id -> trt
+        :returns: a list of TRTs, one for each et_id
         """
-        trt_by_grp = []
-        n = len(self.sm_rlzs)
+        e = len(self.sm_rlzs)
         trts = list(self.gsim_lt.values)
-        for smodel in self.sm_rlzs:
-            for grp_id in self.grp_ids(smodel.ordinal):
-                trt_by_grp.append((grp_id, trts[grp_id // n]))
-        return dict(sorted(trt_by_grp))
+        return [trts[et_id // e] for et_id in range(e*len(trts))]
 
     @property
     def seed(self):
@@ -1199,19 +1230,27 @@ class FullLogicTree(object):
         """
         return self.source_model_lt.num_samples
 
-    def get_trti_eri(self, grp_id):
+    @property
+    def sampling_method(self):
+        """
+        :returns: the source_model_lt ``sampling_method`` parameter
+        """
+        return self.source_model_lt.sampling_method
+
+    def get_trti_eri(self, et_id):
         """
         :returns: (trti, eri)
         """
-        return divmod(grp_id, len(self.sm_rlzs))
+        return divmod(et_id, len(self.sm_rlzs))
 
-    def get_grp_id(self, trt, eri):
+    def get_et_id(self, trt, eri):
         """
-        :returns: grp_id
+        :returns: et_id
         """
-        return self.trti[trt] * len(self.sm_rlzs) + int(eri)
+        gid = self.trti[trt] * len(self.sm_rlzs) + int(eri)
+        return gid
 
-    def grp_ids(self, eri):
+    def et_ids(self, eri):
         """
         :param eri: effective realization index
         :returns: array of T group IDs, being T the number of TRTs
@@ -1220,49 +1259,47 @@ class FullLogicTree(object):
         ns = len(self.sm_rlzs)
         return eri + numpy.arange(nt) * ns
 
-    def get_samples_by_grp(self):
-        """
-        :returns: a dictionary grp_id -> source_model.samples
-        """
-        return {grp_id: sm.samples for sm in self.sm_rlzs
-                for grp_id in self.grp_ids(sm.ordinal)}
-
     def gsim_by_trt(self, rlz):
         """
         :returns: a dictionary trt->gsim for the given realization
         """
         return dict(zip(self.gsim_lt.values, rlz.gsim_rlz.value))
 
-    def get_rlzs(self, eri):
-        """
-        :returns: a list of LtRealization objects
-        """
-        rlzs = []
-        sm = self.sm_rlzs[eri]
-        if self.num_samples:
-            gsim_rlzs = self.gsim_lt.sample(sm.samples, self.seed + sm.ordinal)
-        elif hasattr(self, 'gsim_rlzs'):  # cache
-            gsim_rlzs = self.gsim_rlzs
-        else:
-            self.gsim_rlzs = gsim_rlzs = get_effective_rlzs(self.gsim_lt)
-        for i, gsim_rlz in enumerate(gsim_rlzs):
-            weight = sm.weight * gsim_rlz.weight
-            rlz = LtRealization(sm.offset + i, sm.lt_path, gsim_rlz, weight)
-            rlzs.append(rlz)
-        return rlzs
-
     def get_realizations(self):
         """
         :returns: the complete list of LtRealizations
         """
-        rlzs = sum((self.get_rlzs(sm.ordinal) for sm in self.sm_rlzs), [])
+        rlzs = []
+        self._gsims_by_trt = AccumDict(accum=set())  # trt -> gsims
+        if self.num_samples:  # sampling
+            sm_rlzs = []
+            for sm_rlz in self.sm_rlzs:
+                sm_rlzs.extend([sm_rlz] * sm_rlz.samples)
+            gsim_rlzs = self.gsim_lt.sample(self.num_samples, self.seed + 1,
+                                            self.sampling_method)
+            for t, trt in enumerate(self.gsim_lt.values):
+                self._gsims_by_trt[trt].update(g.value[t] for g in gsim_rlzs)
+            for i, gsim_rlz in enumerate(gsim_rlzs):
+                rlz = LtRealization(i, sm_rlzs[i].lt_path, gsim_rlz,
+                                    sm_rlzs[i].weight * gsim_rlz.weight)
+                rlzs.append(rlz)
+        else:  # full enumeration
+            gsim_rlzs = list(self.gsim_lt)
+            self._gsims_by_trt = self.gsim_lt.values
+            i = 0
+            for sm_rlz in self.sm_rlzs:
+                for gsim_rlz in gsim_rlzs:
+                    rlz = LtRealization(i, sm_rlz.lt_path, gsim_rlz,
+                                        sm_rlz.weight * gsim_rlz.weight)
+                    rlzs.append(rlz)
+                    i += 1
         assert rlzs, 'No realizations found??'
-        if self.num_samples:
+        if self.num_samples and self.sampling_method.startswith('early_'):
             assert len(rlzs) == self.num_samples, (len(rlzs), self.num_samples)
             for rlz in rlzs:
                 for k in rlz.weight.dic:
                     rlz.weight.dic[k] = 1. / self.num_samples
-        else:
+        else:  # keep the weights
             tot_weight = sum(rlz.weight for rlz in rlzs)
             if not tot_weight.is_one():
                 # this may happen for rounding errors; we ensure the sum of
@@ -1271,47 +1308,79 @@ class FullLogicTree(object):
                     rlz.weight = rlz.weight / tot_weight
         return rlzs
 
-    def get_rlzs_by_gsim(self, grp_id):
+    def get_rlzs_by_eri(self):
         """
-        :returns: a dictionary gsim -> rlzs
+        :returns: a dict eri -> rlzs
         """
-        trti, eri = divmod(grp_id, len(self.sm_rlzs))
-        rlzs_by_gsim = AccumDict(accum=[])
-        for rlz in self.get_rlzs(eri):
-            rlzs_by_gsim[rlz.gsim_rlz.value[trti]].append(rlz.ordinal)
-        return {gsim: U32(rlzs) for gsim, rlzs in sorted(rlzs_by_gsim.items())}
+        smltpath = operator.attrgetter('sm_lt_path')
+        eri_by_ltp = self.get_eri_by_ltp()
+        rlzs = self.get_realizations()
+        dic = {eri_by_ltp['~'.join(ltp)]: rlzs for ltp, rlzs in groupby(
+            rlzs, smltpath).items()}
+        return dic
+
+    def get_rlzs_by_gsim(self, et_id):
+        """
+        :returns: a dictionary gsim -> array of rlz indices
+        """
+        if not hasattr(self, '_rlzs_by_grp'):
+            eri_by_ltp = self.get_eri_by_ltp()
+            rlzs = self.get_realizations()
+            acc = AccumDict(accum=AccumDict(accum=[]))  # et_id->gsim->rlzs
+            for sm in self.sm_rlzs:
+                for gid in self.et_ids(sm.ordinal):
+                    trti, eri = divmod(gid, len(self.sm_rlzs))
+                    for rlz in rlzs:
+                        idx = eri_by_ltp['~'.join(rlz.sm_lt_path)]
+                        if idx == eri:
+                            acc[gid][rlz.gsim_rlz.value[trti]].append(
+                                rlz.ordinal)
+            self._rlzs_by_grp = {}
+            for gid, dic in acc.items():
+                self._rlzs_by_grp[gid] = {
+                    gsim: U32(rlzs) for gsim, rlzs in sorted(dic.items())}
+        return self._rlzs_by_grp[et_id]
 
     def get_rlzs_by_gsim_grp(self):
         """
-        :returns: a dictionary grp_id -> gsim -> rlzs
+        :returns: a dictionary et_id -> gsim -> rlzs
         """
         dic = {}
         for sm in self.sm_rlzs:
-            for grp_id in self.grp_ids(sm.ordinal):
-                dic[grp_id] = self.get_rlzs_by_gsim(grp_id)
+            for et_id in self.et_ids(sm.ordinal):
+                dic[et_id] = self.get_rlzs_by_gsim(et_id)
         return dic
 
     def get_rlzs_by_grp(self):
         """
-        :returns: a dictionary grp_id -> [rlzis, ...]
+        :returns: a dictionary et_id -> [rlzis, ...]
         """
         dic = {}
         for sm in self.sm_rlzs:
-            for grp_id in self.grp_ids(sm.ordinal):
-                grp = 'grp-%02d' % grp_id
-                dic[grp] = list(self.get_rlzs_by_gsim(grp_id).values())
-        return dic  # grp_id -> lists of rlzi
+            for et_id in self.et_ids(sm.ordinal):
+                grp = 'grp-%02d' % et_id
+                dic[grp] = list(self.get_rlzs_by_gsim(et_id).values())
+        return {et_id: dic[et_id] for et_id in sorted(dic)}
 
-    def __getnewargs__(self):
-        # with this FullLogicTree instances will be unpickled correctly
-        return self.seed, self.num_samples, self.sm_rlzs
+    def get_rlzs_by_gsim_list(self, list_of_et_ids):
+        """
+        :returns: a list of dictionaries rlzs_by_gsim, one for each grp_id
+        """
+        out = []
+        for grp_id, et_ids in enumerate(list_of_et_ids):
+            dic = AccumDict(accum=[])
+            for et_id in et_ids:
+                for gsim, rlzs in self.get_rlzs_by_gsim(et_id).items():
+                    dic[gsim].extend(rlzs)
+            out.append(dic)
+        return out
 
     def __toh5__(self):
         # save full_lt/sm_data in the datastore
         sm_data = []
         for sm in self.sm_rlzs:
-            sm_data.append((sm.value, sm.weight, '_'.join(sm.lt_path),
-                            sm.samples, sm.offset))
+            sm_data.append((sm.value, sm.weight, '~'.join(sm.lt_path),
+                            sm.samples))
         return (dict(
             source_model_lt=self.source_model_lt,
             gsim_lt=self.gsim_lt,
@@ -1327,10 +1396,9 @@ class FullLogicTree(object):
         self.gsim_lt = dic['gsim_lt']
         self.sm_rlzs = []
         for sm_id, rec in enumerate(sm_data):
-            path = tuple(str(decode(rec['path'])).split('_'))
+            path = tuple(str(decode(rec['path'])).split('~'))
             sm = Realization(
-                rec['name'], rec['weight'], sm_id, path,
-                rec['samples'], rec['offset'])
+                rec['name'], rec['weight'], sm_id, path, rec['samples'])
             self.sm_rlzs.append(sm)
 
     def get_num_rlzs(self, sm_rlz=None):
@@ -1344,41 +1412,46 @@ class FullLogicTree(object):
             return sm_rlz.samples
         return self.gsim_lt.get_num_paths()
 
+    def get_num_potential_paths(self):
+        """
+         :returns: the number of potential realizations
+        """
+        return self.gsim_lt.get_num_paths() * self.source_model_lt.num_paths
+
     @property
     def rlzs(self):
         """
         :returns: an array of realizations
         """
-        tups = [(r.ordinal, r.pid, r.weight['weight'])
-                for r in self.get_realizations()]
+        sh1 = self.source_model_lt.shortener
+        sh2 = self.gsim_lt.shortener
+        tups = []
+        for r in self.get_realizations():
+            path = '%s~%s' % (shorten(r.sm_lt_path, sh1),
+                              shorten(r.gsim_rlz.lt_path, sh2))
+            tups.append((r.ordinal, path, r.weight['weight']))
         return numpy.array(tups, rlz_dt)
 
     def get_gsims_by_trt(self):
         """
         :returns: a dictionary trt -> sorted gsims
         """
-        if self.num_samples:
-            gsims_by_trt = AccumDict(accum=set())
-            for sm in self.sm_rlzs:
-                rlzs = self.gsim_lt.sample(sm.samples, self.seed + sm.ordinal)
-                for t, trt in enumerate(self.gsim_lt.values):
-                    gsims_by_trt[trt].update([rlz.value[t] for rlz in rlzs])
-        else:
-            gsims_by_trt = self.gsim_lt.values
-        return {trt: sorted(gsims) for trt, gsims in gsims_by_trt.items()}
+        if not hasattr(self, '_gsims_by_trt'):
+            self.get_realizations()
+        return {trt: sorted(gs) for trt, gs in self._gsims_by_trt.items()}
 
     def get_sm_by_grp(self):
         """
-        :returns: a dictionary grp_id -> sm_id
+        :returns: a dictionary et_id -> sm_id
         """
-        return {grp_id: sm.ordinal for sm in self.sm_rlzs
-                for grp_id in self.grp_ids(sm.ordinal)}
+        return {et_id: sm.ordinal for sm in self.sm_rlzs
+                for et_id in self.et_ids(sm.ordinal)}
 
     def __repr__(self):
         info_by_model = {}
         for sm in self.sm_rlzs:
             info_by_model[sm.lt_path] = (
-                '_'.join(map(decode, sm.lt_path)),
+                '~'.join(map(decode, sm.lt_path)),
                 decode(sm.value), sm.weight, self.get_num_rlzs(sm))
         summary = ['%s, %s, weight=%s: %d realization(s)' % ibm
                    for ibm in info_by_model.values()]
