@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2010-2020 GEM Foundation
+# Copyright (C) 2010-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -139,10 +139,6 @@ def handle(job_id, log_level='info', log_file=None):
     try:
         yield
     finally:
-        # sanity check to make sure that the logging on file is working
-        if (log_file and log_file != os.devnull and
-                os.path.getsize(log_file) == 0):
-            logging.root.warn('The log file %s is empty!?' % log_file)
         for handler in handlers:
             logging.root.removeHandler(handler)
 
