@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2010-2020 GEM Foundation
+# Copyright (C) 2010-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -17,10 +17,11 @@
 # along with OpenQuake. If not, see <http://www.gnu.org/licenses/>.
 
 import os
+import csv
 import tempfile
 import numpy  # this is needed by the doctests, don't remove it
+import pandas
 from openquake.baselib.node import scientificformat
-from openquake.baselib.python3compat import encode
 
 FIVEDIGITS = '%.5E'
 
@@ -91,16 +92,16 @@ def extract_from(data, fields):
     return data
 
 
-def _header(fields, sep, renamedict):
+def _header(fields, renamedict):
     if renamedict:
         fields = [renamedict.get(f, f) for f in fields]
-    return encode(sep.join(fields) + '\n')
+    return fields
 
 
-def write_csv(dest, data, sep=',', fmt='%.6E', header=None, comment=None,
+def write_csv(dest, data, sep=',', fmt='%.6E', header=(), comment=None,
               renamedict=None):
     """
-    :param dest: None, file, filename or io.BytesIO instance
+    :param dest: None, file, filename or io.StringIO instance
     :param data: array to save
     :param sep: separator to use (default comma)
     :param fmt: formatting string (default '%12.8E')
@@ -120,8 +121,9 @@ def write_csv(dest, data, sep=',', fmt='%.6E', header=None, comment=None,
         # it must be closed by client code
         close = False
     elif not hasattr(dest, 'getvalue'):
-        # not a BytesIO, assume dest is a filename
-        dest = open(dest, 'wb')
+        # assume dest is a filename
+        dest = open(dest, 'w')
+    w = csv.writer(dest, delimiter=sep)
     try:
         # see if data is a composite numpy array
         data.dtype.fields
@@ -131,22 +133,16 @@ def write_csv(dest, data, sep=',', fmt='%.6E', header=None, comment=None,
     else:
         autoheader = build_header(data.dtype)
 
-    nfields = len(autoheader) or len(data[0])
+    nfields = len(autoheader) or len(header) or len(data[0])
     if comment:
-        if '"' in comment:  # double quote
-            comment = comment.replace('"', '""')
-        com = '#%s"%s"\n' % (sep * (nfields - 1), comment)
-        dest.write(encode(com))
+        w.writerow(['#'] + [''] * (nfields - 2) + [comment])
 
     someheader = header or autoheader
     if header != 'no-header' and someheader:
-        dest.write(_header(someheader, sep, renamedict))
+        w.writerow(_header(someheader, renamedict))
 
     def format(val):
-        col = scientificformat(val, fmt)
-        if sep in col and not col.startswith('"'):
-            return '"%s"' % col
-        return col
+        return scientificformat(val, fmt)
 
     if autoheader:
         all_fields = [col.split(':', 1)[0].split('~')
@@ -159,12 +155,12 @@ def write_csv(dest, data, sep=',', fmt='%.6E', header=None, comment=None,
                     row.append('%.5f' % val)
                 else:
                     row.append(format(val))
-            dest.write(_header(row, sep, renamedict))
+            w.writerow(_header(row, renamedict))
     else:
         for row in data:
-            dest.write(encode(sep.join(format(col) for col in row) + '\n'))
+            w.writerow([format(col) for col in row])
     if hasattr(dest, 'getvalue'):
-        return dest.getvalue()[:-1]  # a newline is strangely added
+        return
     elif close:
         dest.close()
     return dest.name
@@ -179,22 +175,33 @@ class CsvWriter(object):
         self.fmt = fmt
         self.fnames = set()
 
-    def save(self, data, fname, header=None, comment=None, renamedict=None):
+    def save(self, data, fname, header=(), comment=None, renamedict=None):
         """
         Save data on fname.
 
-        :param data: numpy array or list of lists
+        :param data: numpy array, list of lists or pandas DataFrame
         :param fname: path name
         :param header: header to use
         :param comment: optional dictionary to be converted in a comment
         :param renamedict: a dictionary for renaming the columns
         """
-        write_csv(fname, data, self.sep, self.fmt, header, comment, renamedict)
+        if isinstance(data, pandas.DataFrame):
+            if comment is None:
+                data.to_csv(fname, index=False, float_format=self.fmt,
+                            line_terminator='\r\n')
+            else:
+                write_csv(fname, [], self.sep, self.fmt, list(data.columns),
+                          comment=comment)
+                data.to_csv(fname, index=False, float_format=self.fmt,
+                            line_terminator='\r\n', header=False, mode='a')
+        else:
+            write_csv(fname, data, self.sep, self.fmt, header, comment,
+                      renamedict)
         self.fnames.add(getattr(fname, 'name', fname))
 
     def save_block(self, data, dest):
         """
-        Save data on dest, which is file open in 'a' mode
+        Save data on dest, which is a file open in 'a' mode
         """
         write_csv(dest, data, self.sep, self.fmt, 'no-header')
 
