@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2020 GEM Foundation
+# Copyright (C) 2012-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -24,6 +24,7 @@ import time
 import numpy
 import scipy.stats
 
+from openquake.baselib.general import AccumDict
 from openquake.hazardlib.const import StdDev
 from openquake.hazardlib.gsim.base import ContextMaker
 from openquake.hazardlib.gsim.multi import MultiGMPE
@@ -99,76 +100,92 @@ class GmfComputer(object):
     # a matrix of size (I, N, E) is returned, where I is the number of
     # IMTs, N the number of affected sites and E the number of events. The
     # seed is extracted from the underlying rupture.
-    def __init__(self, rupture, sitecol, imts, cmaker,
+    def __init__(self, rupture, sitecol, cmaker,
                  truncation_level=None, correlation_model=None,
-                 amplifier=None):
+                 amplifier=None, sec_perils=()):
         if len(sitecol) == 0:
             raise ValueError('No sites')
-        elif len(imts) == 0:
+        elif len(cmaker.imtls) == 0:
             raise ValueError('No IMTs')
         elif len(cmaker.gsims) == 0:
             raise ValueError('No GSIMs')
-        self.rupture = rupture
-        self.imts = [from_string(imt) for imt in imts]
+        self.imts = [from_string(imt) for imt in cmaker.imtls]
         self.gsims = sorted(cmaker.gsims)
         self.truncation_level = truncation_level
         self.correlation_model = correlation_model
         self.amplifier = amplifier
-        # `rupture` can be an EBRupture instance
-        if hasattr(rupture, 'srcidx'):
-            self.srcidx = rupture.srcidx  # the source the rupture comes from
-            self.e0 = rupture.e0
+        self.sec_perils = sec_perils
+        # `rupture` is an EBRupture instance in the engine
+        if hasattr(rupture, 'source_id'):
+            self.ebrupture = rupture
+            self.source_id = rupture.source_id  # the underlying source
             rupture = rupture.rupture  # the underlying rupture
-        else:
-            self.srcidx = '?'
-            self.e0 = 0
-        try:
-            self.sctx, self.dctx = rupture.sctx, rupture.dctx
-        except AttributeError:
-            self.sctx, self.dctx = cmaker.make_contexts(sitecol, rupture)
+        else:  # in the hazardlib tests
+            self.source_id = '?'
+        self.seed = rupture.rup_id
+        self.rctx, self.sctx, self.dctx = cmaker.make_contexts(
+            sitecol, rupture)
         self.sids = self.sctx.sids
         if correlation_model:  # store the filtered sitecol
             self.sites = sitecol.complete.filtered(self.sids)
+        if truncation_level is None:
+            self.distribution = scipy.stats.norm()
+        elif truncation_level == 0:
+            self.distribution = None
+        else:
+            assert truncation_level > 0, truncation_level
+            self.distribution = scipy.stats.truncnorm(
+                - truncation_level, truncation_level)
 
     def compute_all(self, min_iml, rlzs_by_gsim, sig_eps=None):
         """
-        :returns: [(sid, eid, gmv), ...], dt
+        :returns: (dict with fields eid, sid, gmv_...), dt
         """
         t0 = time.time()
-        rup = self.rupture
         sids = self.sids
-        eids_by_rlz = rup.get_eids_by_rlz(rlzs_by_gsim)
-        data = []
+        eids_by_rlz = self.ebrupture.get_eids_by_rlz(rlzs_by_gsim)
+        mag = self.ebrupture.rupture.mag
+        data = AccumDict(accum=[])
         for gs, rlzs in rlzs_by_gsim.items():
-            num_events = sum(len(eids_by_rlz[rlzi]) for rlzi in rlzs)
+            num_events = sum(len(eids_by_rlz[rlz]) for rlz in rlzs)
+            if num_events == 0:  # it may happen
+                continue
             # NB: the trick for performance is to keep the call to
-            # compute.compute outside of the loop over the realizations
+            # .compute outside of the loop over the realizations;
             # it is better to have few calls producing big arrays
             array, sig, eps = self.compute(gs, num_events)
+            M, N, E = array.shape
+            for n in range(N):
+                for e in range(E):
+                    if (array[:, n, e] < min_iml).all():
+                        array[:, n, e] = 0
             array = array.transpose(1, 0, 2)  # from M, N, E to N, M, E
-            for i, miniml in enumerate(min_iml):  # gmv < minimum
-                arr = array[:, i, :]
-                arr[arr < miniml] = 0
             n = 0
-            for rlzi in rlzs:
-                eids = eids_by_rlz[rlzi] + self.e0
-                e = len(eids)
+            for rlz in rlzs:
+                eids = eids_by_rlz[rlz]
                 for ei, eid in enumerate(eids):
-                    gmf = array[:, :, n + ei]  # shape (N, M)
-                    tot = gmf.sum(axis=0)  # shape (M,)
-                    if not tot.sum():
-                        continue
+                    gmfa = array[:, :, n + ei]  # shape (N, M)
                     if sig_eps is not None:
-                        tup = tuple([eid, rlzi] + list(sig[:, n + ei]) +
+                        tup = tuple([eid, rlz] + list(sig[:, n + ei]) +
                                     list(eps[:, n + ei]))
                         sig_eps.append(tup)
-                    for sid, gmv in zip(sids, gmf):
-                        if gmv.sum():
-                            data.append((sid, eid, gmv))
-                n += e
-        m = (len(min_iml),)
-        d = numpy.array(data, [('sid', U32), ('eid', U32), ('gmv', (F32, m))])
-        return d, time.time() - t0
+                    items = []
+                    for sp in self.sec_perils:
+                        o = sp.compute(mag, zip(self.imts, gmfa.T), self.sctx)
+                        for outkey, outarr in zip(sp.outputs, o):
+                            items.append((outkey, outarr))
+                    for i, gmv in enumerate(gmfa):
+                        data['sid'].append(sids[i])
+                        data['eid'].append(eid)
+                        data['rlz'].append(rlz)
+                        for m in range(M):
+                            data[f'gmv_{m}'].append(gmv[m])
+                        for outkey, outarr in items:
+                            data[outkey].append(outarr[i])
+                        # gmv can be zero due to the minimum_intensity, coming
+                        # from the job.ini or from the vulnerability functions
+                n += len(eids)
+        return data, time.time() - t0
 
     def compute(self, gsim, num_events):
         """
@@ -179,10 +196,10 @@ class GmfComputer(object):
             two arrays with shape (num_imts, num_events): sig for stddev_inter
             and eps for the random part
         """
-        numpy.random.seed(self.rupture.rup_id)
         result = numpy.zeros((len(self.imts), len(self.sids), num_events), F32)
         sig = numpy.zeros((len(self.imts), num_events), F32)
         eps = numpy.zeros((len(self.imts), num_events), F32)
+        numpy.random.seed(self.seed)
         for imti, imt in enumerate(self.imts):
             if isinstance(gsim, MultiGMPE):
                 gs = gsim[str(imt)]  # MultiGMPE
@@ -190,47 +207,39 @@ class GmfComputer(object):
                 gs = gsim  # regular GMPE
             try:
                 result[imti], sig[imti], eps[imti] = self._compute(
-                    None, gs, num_events, imt)
+                     gs, num_events, imt)
             except Exception as exc:
-                raise exc.__class__(
-                    '%s for %s, %s, srcidx=%s' % (exc, gs, imt, self.srcidx)
+                raise RuntimeError(
+                    '(%s, %s, source_id=%r) %s: %s' %
+                    (gs, imt, self.source_id.decode('utf8'),
+                     exc.__class__.__name__, exc)
                 ).with_traceback(exc.__traceback__)
+        if self.amplifier:
+            self.amplifier.amplify_gmfs(
+                self.sctx.ampcode, result, self.imts, self.seed)
         return result, sig, eps
 
-    def _compute(self, seed, gsim, num_events, imt):
+    def _compute(self, gsim, num_events, imt):
         """
-        :param seed: a random seed or None if the seed is already set
         :param gsim: a GSIM instance
         :param num_events: the number of seismic events
         :param imt: an IMT instance
         :returns: (gmf(num_sites, num_events), stddev_inter(num_events),
                    epsilons(num_events))
         """
-        rctx = getattr(self.rupture, 'rupture', self.rupture)
-        if seed is not None:
-            numpy.random.seed(seed)
         dctx = self.dctx.roundup(gsim.minimum_distance)
-        if self.truncation_level == 0:
+        if self.distribution is None:
             if self.correlation_model:
                 raise ValueError('truncation_level=0 requires '
                                  'no correlation model')
             mean, _stddevs = gsim.get_mean_and_stddevs(
-                self.sctx, rctx, dctx, imt, stddev_types=[])
-            mean = to_imt_unit_values(mean, imt)
-            mean.shape += (1, )
-            mean = mean.repeat(num_events, axis=1)
-            if self.amplifier:
-                self.amplifier.amplify_gmfs(self.sctx.ampcode, mean, str(imt))
-            return (mean,
+                self.sctx, self.rctx, dctx, imt, stddev_types=[])
+            gmf = to_imt_unit_values(mean, imt)
+            gmf.shape += (1, )
+            gmf = gmf.repeat(num_events, axis=1)
+            return (gmf,
                     numpy.zeros(num_events, F32),
                     numpy.zeros(num_events, F32))
-        elif self.truncation_level is None:
-            distribution = scipy.stats.norm()
-        else:
-            assert self.truncation_level > 0, self.truncation_level
-            distribution = scipy.stats.truncnorm(
-                - self.truncation_level, self.truncation_level)
-
         num_sids = len(self.sids)
         if gsim.DEFINED_FOR_STANDARD_DEVIATION_TYPES == {StdDev.TOTAL}:
             # If the GSIM provides only total standard deviation, we need
@@ -242,25 +251,25 @@ class GmfComputer(object):
                     self.correlation_model, gsim)
 
             mean, [stddev_total] = gsim.get_mean_and_stddevs(
-                self.sctx, rctx, dctx, imt, [StdDev.TOTAL])
+                self.sctx, self.rctx, dctx, imt, [StdDev.TOTAL])
             stddev_total = stddev_total.reshape(stddev_total.shape + (1, ))
             mean = mean.reshape(mean.shape + (1, ))
 
             total_residual = stddev_total * rvs(
-                distribution, num_sids, num_events)
+                self.distribution, num_sids, num_events)
             gmf = to_imt_unit_values(mean + total_residual, imt)
             stdi = numpy.nan
             epsilons = numpy.empty(num_events, F32)
             epsilons.fill(numpy.nan)
         else:
             mean, [stddev_inter, stddev_intra] = gsim.get_mean_and_stddevs(
-                self.sctx, rctx, dctx, imt,
+                self.sctx, self.rctx, dctx, imt,
                 [StdDev.INTER_EVENT, StdDev.INTRA_EVENT])
             stddev_intra = stddev_intra.reshape(stddev_intra.shape + (1, ))
             stddev_inter = stddev_inter.reshape(stddev_inter.shape + (1, ))
             mean = mean.reshape(mean.shape + (1, ))
             intra_residual = stddev_intra * rvs(
-                distribution, num_sids, num_events)
+                self.distribution, num_sids, num_events)
 
             if self.correlation_model is not None:
                 intra_residual = self.correlation_model.apply_correlation(
@@ -269,14 +278,12 @@ class GmfComputer(object):
                 if len(sh) == 1:  # a vector
                     intra_residual = intra_residual.reshape(sh + (1,))
 
-            epsilons = rvs(distribution, num_events)
+            epsilons = rvs(self.distribution, num_events)
             inter_residual = stddev_inter * epsilons
 
             gmf = to_imt_unit_values(
                 mean + intra_residual + inter_residual, imt)
             stdi = stddev_inter.max(axis=0)
-        if self.amplifier:
-            self.amplifier.amplify_gmfs(self.sctx.ampcode, gmf, str(imt))
         return gmf, stdi, epsilons
 
 
@@ -326,9 +333,10 @@ def ground_motion_fields(rupture, sites, imts, gsim, truncation_level,
         for all sites in the collection. First dimension represents
         sites and second one is for realizations.
     """
-    cmaker = ContextMaker(rupture.tectonic_region_type, [gsim])
+    cmaker = ContextMaker(rupture.tectonic_region_type, [gsim],
+                          dict(imtls={str(imt): [1] for imt in imts}))
     rupture.rup_id = seed
-    gc = GmfComputer(rupture, sites, [str(imt) for imt in imts],
-                     cmaker, truncation_level, correlation_model)
+    gc = GmfComputer(rupture, sites, cmaker, truncation_level,
+                     correlation_model)
     res, _sig, _eps = gc.compute(gsim, realizations)
     return {imt: res[imti] for imti, imt in enumerate(gc.imts)}
