@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2020 GEM Foundation
+# Copyright (C) 2012-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -60,18 +60,37 @@ class Polygon(object):
         self._projection = None
         self._polygon2d = None
 
+    @property
+    def coords(self):
+        """
+        Coordinates of the polygon as a linear ring, rounded to 5 digits
+        """
+        lons = numpy.round(self.lons, 5)
+        lats = numpy.round(self.lats, 5)
+        pairs = list(zip(lons, lats))
+        return pairs + [pairs[0]]
+
     @property
     def wkt(self):
         """
         Generate WKT (Well-Known Text) to represent this polygon.
         """
-        pairs = ['%s %s' % (lon, lat)
+        pairs = ['%.5f %.5f' % (lon, lat)
                  for lon, lat in zip(self.lons, self.lats)]
         # the polygon must form a closed loop; first and last coord pairs
         # are the same
         pairs.append(pairs[0])
         return 'POLYGON((%s))' % ', '.join(pairs)
 
+    @property
+    def centroid(self):
+        """
+        :returns: the centroid of the underlying 2D polygon
+        """
+        if self._polygon2d is None:
+            self._init_polygon2d()
+        return self._polygon2d.centroid
+
     @classmethod
     def from_wkt(cls, wkt_string):
         """
