@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2020 GEM Foundation
+# Copyright (C) 2012-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -27,6 +27,7 @@ import warnings
 import functools
 import numpy
 from scipy.special import ndtr
+from scipy.stats import norm
 
 from openquake.baselib.general import DeprecationWarning
 from openquake.hazardlib import imt as imt_module
@@ -83,123 +84,14 @@ def gsim_imt_dt(sorted_gsims, sorted_imts):
     return numpy.dtype([(str(gsim), imt_dt) for gsim in sorted_gsims])
 
 
-def get_mean_std(sctx, rctx, dctx, imts, gsims):
-    """
-    :returns: an array of shape (2, N, M, G) with means and stddevs
-    """
-    N = len(sctx.sids)
-    M = len(imts)
-    G = len(gsims)
-    arr = numpy.zeros((2, N, M, G))
-    num_tables = CoeffsTable.num_instances
-    for g, gsim in enumerate(gsims):
-        d = dctx.roundup(gsim.minimum_distance)
-        for m, imt in enumerate(imts):
-            mean, [std] = gsim.get_mean_and_stddevs(sctx, rctx, d, imt,
-                                                    [const.StdDev.TOTAL])
-            arr[0, :, m, g] = mean
-            arr[1, :, m, g] = std
-            if CoeffsTable.num_instances > num_tables:
-                raise RuntimeError('Instantiating CoeffsTable inside '
-                                   '%s.get_mean_and_stddevs' %
-                                   gsim.__class__.__name__)
-    return arr
-
-
-def get_poes(mean_std, loglevels, truncation_level, gsims=()):
-    """
-    Calculate and return probabilities of exceedance (PoEs) of one or more
-    intensity measure levels (IMLs) of one intensity measure type (IMT)
-    for one or more pairs "site -- rupture".
-
-    :param mean_std:
-        An array of shape (2, N, M, G) with mean and standard deviation for
-        the current intensity measure type
-    :param loglevels:
-        A DictArray imt -> logs of intensity measure levels
-    :param truncation_level:
-        Can be ``None``, which means that the distribution of intensity
-        is treated as Gaussian distribution with possible values ranging
-        from minus infinity to plus infinity.
-
-        When set to zero, the mean intensity is treated as an exact
-        value (standard deviation is not even computed for that case)
-        and resulting array contains 0 in places where IMT is strictly
-        lower than the mean value of intensity and 1.0 where IMT is equal
-        or greater.
-
-        When truncation level is positive number, the intensity
-        distribution is processed as symmetric truncated Gaussian with
-        range borders being ``mean - truncation_level * stddev`` and
-        ``mean + truncation_level * stddev``. That is, the truncation
-        level expresses how far the range borders are from the mean
-        value and is defined in units of sigmas. The resulting PoEs
-        for that mode are values of complementary cumulative distribution
-        function of that truncated Gaussian applied to IMLs.
-
-    :returns:
-        array of PoEs of shape (N, L, G)
-
-    :raises ValueError:
-        If truncation level is not ``None`` and neither non-negative
-        float number, and if ``imts`` dictionary contain wrong or
-        unsupported IMTs (see :attr:`DEFINED_FOR_INTENSITY_MEASURE_TYPES`).
-    """
-    if truncation_level is not None and truncation_level < 0:
-        raise ValueError('truncation level must be zero, positive number '
-                         'or None')
-    if len(gsims):
-        assert mean_std.shape[-1] == len(gsims)
-    tl = truncation_level
-    if any(hasattr(gsim, 'weights_signs') for gsim in gsims):
-        # implement average get_poes for the nshmp_2014 model
-        shp = list(mean_std[0].shape)  # (N, M, G)
-        shp[1] = len(loglevels.array)  # L
-        arr = numpy.zeros(shp)
-        for g, gsim in enumerate(gsims):
-            if hasattr(gsim, 'weights_signs'):
-                outs = []
-                weights, signs = zip(*gsim.weights_signs)
-                for s in signs:
-                    ms = numpy.array(mean_std[:, :, :, g])  # make a copy
-                    for m in range(len(loglevels)):
-                        ms[0, :, m] += s * gsim.adjustment
-                    outs.append(_get_poes(ms, loglevels, tl, squeeze=1))
-                arr[:, :, g] = numpy.average(outs, weights=weights, axis=0)
-            else:
-                ms = mean_std[:, :, :, g]
-                arr[:, :, g] = _get_poes(ms, loglevels, tl, squeeze=1)
-        return arr
-    elif any("mixture_model" in gsim.kwargs for gsim in gsims):
-        shp = list(mean_std[0].shape)  # (N, M, G)
-        shp[1] = len(loglevels.array)  # L
-        arr = numpy.zeros(shp)
-        for g, gsim in enumerate(gsims):
-            if "mixture_model" in gsim.kwargs:
-                for fact, wgt in zip(
-                    gsim.kwargs["mixture_model"]["factors"],
-                    gsim.kwargs["mixture_model"]["weights"]):
-                    mean_stdi = numpy.copy(mean_std)
-                    mean_stdi[1, :, :, :] *= fact
-                    arr[:, :, g] += (wgt * _get_poes(mean_stdi, loglevels, tl,
-                                                     squeeze=1))
-            else:
-                ms = mean_std[:, :, :, g]
-                arr[:, :, g] = _get_poes(ms, loglevels, tl, squeeze=1)
-        return arr
-    else:
-        # regular case
-        return _get_poes(mean_std, loglevels, truncation_level)
-
-
 # this is the critical function for the performance of the classical calculator
 # it is dominated by memory allocations (i.e. _truncnorm_sf is ultra-fast)
 # the only way to speedup is to reduce the maximum_distance, then the array
-# will become shorted in the N dimension (number of affected sites)
-def _get_poes(mean_std, loglevels, truncation_level, squeeze=False):
-    mean, stddev = mean_std  # shape (N, M, G) each
-    N, L, G = len(mean), len(loglevels.array), mean.shape[-1]
-    out = numpy.zeros((N, L) if squeeze else (N, L, G))
+# will become shorter in the N dimension (number of affected sites), or to
+# collapse the ruptures, then _get_poes will be called less times
+def _get_poes(mean_std, loglevels, truncation_level):
+    mean, stddev = mean_std  # shape (N, M) each
+    out = numpy.zeros((len(mean), loglevels.size))  # shape (N, L)
     lvl = 0
     for m, imt in enumerate(loglevels):
         for iml in loglevels[imt]:
@@ -211,6 +103,100 @@ def _get_poes(mean_std, loglevels, truncation_level, squeeze=False):
     return _truncnorm_sf(truncation_level, out)
 
 
+def _get_poes_site(mean_std, loglevels, truncation_level, ampfun, ctxs):
+    """
+    NOTE: this works for a single site
+
+    :param mean_std:
+        See :function:`openquake.hazardlib.gsim.base.get_poes`
+    :param loglevels:
+        Intensity measure level per intensity measure type. See
+        :function:`openquake.hazardlib.gsim.base.get_poes`
+    :param truncation_level:
+        The level of truncation of the normal distribution of ground-motion
+        on rock
+    :param ampl:
+        Site amplification function instance of
+        :class:openquake.hazardlib.site_amplification.AmpFunction
+    :param ctxs:
+        Context objects with attributes .mag, .sites, .rrup
+    """
+    # Mean and std of ground motion for the IMTs considered in this analysis
+    # C - Number of contexts
+    # L - Number of intensity measure levels
+    mean, stddev = mean_std  # shape (C, M)
+    C, L = len(mean), loglevels.size
+    for ctx in ctxs:
+        assert len(ctx.sids) == 1  # 1 site
+    M = len(loglevels)
+    L1 = L // M
+
+    # This is the array where we store the output results i.e. poes on soil
+    out_s = numpy.zeros((C, L))
+
+    # `nsamp` is the number of IMLs per IMT used to compute the hazard on rock
+    # while 'L' is total number of ground-motion values
+    nsamp = 40
+
+    # Compute the probability of exceedance for each in intensity
+    # measure type IMT
+    sigma = ampfun.get_max_sigma()
+    mags = [ctx.mag for ctx in ctxs]
+    rrups = [ctx.rrup for ctx in ctxs]
+    ampcode = ctxs[0].sites['ampcode'][0]
+    for m, imt in enumerate(loglevels):
+
+        # Get the values of ground-motion used to compute the probability
+        # of exceedance on soil.
+        soillevels = loglevels[imt]  # shape L1
+
+        # Here we set automatically the IMLs that will be used to compute
+        # the probability of occurrence of GM on rock within discrete
+        # intervals
+        ll = numpy.linspace(min(soillevels) - sigma * 4.,
+                            max(soillevels) + sigma * 4.,
+                            num=nsamp)
+
+        # Calculate for each ground motion interval the probability
+        # of occurrence on rock for all the sites
+        for iml_l, iml_u in zip(ll[:-1], ll[1:]):
+
+            # Set the arguments of the truncated normal distribution
+            # function
+            if truncation_level == 0:
+                out_l = iml_l <= mean[:, m]
+                out_u = iml_u <= mean[:, m]
+            else:
+                out_l = (iml_l - mean[:, m]) / stddev[:, m]
+                out_u = (iml_u - mean[:, m]) / stddev[:, m]
+
+            # Probability of occurrence on rock
+            pocc_rock = (_truncnorm_sf(truncation_level, out_l) -
+                         _truncnorm_sf(truncation_level, out_u))  # shape C
+
+            # Skipping cases where the pocc on rock is negligible
+            if numpy.all(pocc_rock < 1e-10):
+                continue
+
+            # Ground-motion value in the middle of each interval
+            iml_mid = (numpy.exp(iml_l) + numpy.exp(iml_u)) / 2.
+
+            # Get mean and std of the amplification function for this
+            # magnitude, distance and IML
+            median_af, std_af = ampfun.get_mean_std(  # shape C
+                ampcode, imt, iml_mid, mags, rrups)
+
+            # Computing the probability of exceedance of the levels of
+            # ground-motion loglevels on soil
+            logaf = numpy.log(numpy.exp(soillevels) / iml_mid)  # shape L1
+            for l in range(L1):
+                poex_af = 1. - norm.cdf(
+                    logaf[l], numpy.log(median_af), std_af)  # shape C
+                out_s[:, m * L1 + l] += poex_af * pocc_rock  # shape C
+
+    return out_s
+
+
 class MetaGSIM(abc.ABCMeta):
     """
     A metaclass converting set class attributes into frozensets, to avoid
@@ -226,7 +212,12 @@ class MetaGSIM(abc.ABCMeta):
                     if missing:
                         raise ValueError('Unknown distance %s in %s' %
                                          (missing, name))
-        return super().__new__(meta, name, bases, dic)
+        cls = super().__new__(meta, name, bases, dic)
+        ancestors = [vars(ancestor) for ancestor in cls.mro()[1:-1]]
+        if any('get_mean_std1' in ancestor for ancestor in ancestors):
+            if 'get_mean_and_stddevs' in dic and 'get_mean_std1' not in dic:
+                raise TypeError('%s.get_mean_std1 is not defined!' % name)
+        return cls
 
 
 @functools.total_ordering
@@ -242,6 +233,7 @@ class GroundShakingIntensityModel(metaclass=MetaGSIM):
     This class is not intended to be subclassed directly, instead
     the actual GSIMs should subclass :class:`GMPE`
 
+
     Subclasses of both must implement :meth:`get_mean_and_stddevs`
     and all the class attributes with names starting from ``DEFINED_FOR``
     and ``REQUIRES``.
@@ -327,7 +319,6 @@ class GroundShakingIntensityModel(metaclass=MetaGSIM):
     non_verified = False
     experimental = False
     adapted = False
-    get_poes = staticmethod(get_poes)
 
     @classmethod
     def __init_subclass__(cls):
@@ -553,6 +544,104 @@ class GMPE(GroundShakingIntensityModel):
             else:
                 setattr(self, key, val)
 
+    def get_mean_std(self, ctxs, imts):
+        """
+        :returns: an array of shape (2, N, M) with means and stddevs
+        """
+        N = sum(len(ctx.sids) for ctx in ctxs)
+        M = len(imts)
+        arr = numpy.zeros((2, N, M))
+        num_tables = CoeffsTable.num_instances
+        start = 0
+        for ctx in ctxs:
+            stop = start + len(ctx.sids)
+            new = ctx.roundup(self.minimum_distance)
+            for m, imt in enumerate(imts):
+                mean, [std] = self.get_mean_and_stddevs(ctx, ctx, new, imt,
+                                                        [const.StdDev.TOTAL])
+                arr[0, start:stop, m] = mean
+                arr[1, start:stop, m] = std
+                if CoeffsTable.num_instances > num_tables:
+                    raise RuntimeError('Instantiating CoeffsTable inside '
+                                       '%s.get_mean_and_stddevs' %
+                                       self.__class__.__name__)
+            start = stop
+        return arr
+
+    def get_poes(self, mean_std, loglevels, trunclevel, af=None, ctxs=()):
+        """
+        Calculate and return probabilities of exceedance (PoEs) of one or more
+        intensity measure levels (IMLs) of one intensity measure type (IMT)
+        for one or more pairs "site -- rupture".
+
+        :param mean_std:
+            An array of shape (2, N, M) with mean and standard deviations
+            for the sites and intensity measure types
+        :param loglevels:
+            A DictArray imt -> logs of intensity measure levels
+        :param trunclevel:
+            Can be ``None``, which means that the distribution of intensity
+            is treated as Gaussian distribution with possible values ranging
+            from minus infinity to plus infinity.
+
+            When set to zero, the mean intensity is treated as an exact
+            value (standard deviation is not even computed for that case)
+            and resulting array contains 0 in places where IMT is strictly
+            lower than the mean value of intensity and 1.0 where IMT is equal
+            or greater.
+
+            When truncation level is positive number, the intensity
+            distribution is processed as symmetric truncated Gaussian with
+            range borders being ``mean - truncation_level * stddev`` and
+            ``mean + truncation_level * stddev``. That is, the truncation
+            level expresses how far the range borders are from the mean
+            value and is defined in units of sigmas. The resulting PoEs
+            for that mode are values of complementary cumulative distribution
+            function of that truncated Gaussian applied to IMLs.
+        :param af:
+            None or an instance of AmplFunction
+        :param ctxs:
+            Context object used to compute mean_std
+        :returns:
+            array of PoEs of shape (N, L)
+        :raises ValueError:
+            If truncation level is not ``None`` and neither non-negative
+            float number, and if ``imts`` dictionary contain wrong or
+            unsupported IMTs (see :attr:`DEFINED_FOR_INTENSITY_MEASURE_TYPES`).
+        """
+        if trunclevel is not None and trunclevel < 0:
+            raise ValueError('truncation level must be zero, positive number '
+                             'or None')
+        if hasattr(self, 'weights_signs'):
+            outs = []
+            weights, signs = zip(*self.weights_signs)
+            for s in signs:
+                ms = numpy.array(mean_std)  # make a copy
+                for m in range(len(loglevels)):
+                    ms[0, :, m] += s * self.adjustment
+                outs.append(_get_poes(ms, loglevels, trunclevel))
+            arr = numpy.average(outs, weights=weights, axis=0)
+        elif hasattr(self, "mixture_model"):
+            shp = list(mean_std[0].shape)  # (N, M)
+            shp[1] = loglevels.size  # L
+            arr = numpy.zeros(shp)
+            for f, w in zip(self.mixture_model["factors"],
+                            self.mixture_model["weights"]):
+                mean_stdi = numpy.array(mean_std)  # a copy
+                mean_stdi[1] *= f  # multiply stddev by factor
+                arr += w * _get_poes(mean_stdi, loglevels, trunclevel)
+        elif af:  # kernel amplification function
+            arr = _get_poes_site(mean_std, loglevels, trunclevel, af, ctxs)
+        else:  # regular case
+            arr = _get_poes(mean_std, loglevels, trunclevel)
+        imtweight = getattr(self, 'weight', None)  # ImtWeight or None
+        for imt in loglevels:
+            if imtweight and imtweight.dic.get(imt) == 0:
+                # set by the engine when parsing the gsim logictree
+                # when 0 ignore the contribution: see _build_trts_branches
+                arr[:, loglevels(imt)] = 0
+        return arr
+
 
 class CoeffsTable(object):
     r"""
