@@ -1,5 +1,5 @@
 # The Hazard Library
-# Copyright (C) 2012-2020 GEM Foundation
+# Copyright (C) 2012-2021 GEM Foundation
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -62,7 +62,8 @@ class GenericGmpeAvgSA(GMPE):
 
     def __init__(self, gmpe_name, avg_periods, corr_func='none', **kwargs):
 
-        super().__init__(gmpe_name=gmpe_name)
+        super().__init__(gmpe_name=gmpe_name, avg_periods=avg_periods,
+                         corr_func=corr_func, **kwargs)
         self.gmpe = registry[gmpe_name](**kwargs)
         self.set_parameters()
         self.avg_periods = avg_periods
