@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2020 GEM Foundation
+# Copyright (C) 2014-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -99,7 +99,8 @@ class PitilakisEtAl2018(GMPE):
     DEFINED_FOR_REFERENCE_VELOCITY = 800.0
 
     def __init__(self, gmpe_name, reference_velocity=None, **kwargs):
-        super().__init__()
+        super().__init__(gmpe_name=gmpe_name,
+                         reference_velocity=reference_velocity, **kwargs)
         if isinstance(gmpe_name, str):
             self.gmpe = registry[gmpe_name](**kwargs)
         else:
@@ -319,7 +320,7 @@ class Eurocode8Amplification(PitilakisEtAl2018):
     DEFINED_FOR_REFERENCE_VELOCITY = 800.0
 
     def __init__(self, gmpe_name, reference_velocity=800.0, **kwargs):
-        super().__init__(gmpe_name=gmpe_name)
+        super().__init__(gmpe_name=gmpe_name, **kwargs)
         self.rock_vs30 = reference_velocity if reference_velocity else\
             self.DEFINED_FOR_REFERENCE_VELOCITY
         for name in uppernames:
@@ -533,7 +534,7 @@ class SandikkayaDinsever2018(GMPE):
 
     def __init__(self, gmpe_name, reference_velocity=760., region=None,
                  phi_0=None, **kwargs):
-        super().__init__()
+        super().__init__(**kwargs)
         if isinstance(gmpe_name, str):
             self.gmpe = registry[gmpe_name](**kwargs)
         else:
