@@ -17,6 +17,7 @@
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
 
 import copy
+import collections
 import numpy
 
 from openquake.baselib.general import CallableDict
@@ -82,6 +83,13 @@ def incMFD(utype, node, filename):
     return min_mag,  bin_width, ~node.incrementalMFD.occurRates
 
 
+@parse_uncertainty.add('truncatedGRFromSlipAbsolute')
+def trucMFDFromSlip_absolute(utype, node, filename):
+    slip_rate, rigidity = (node.faultActivityData["slipRate"],
+                           node.faultActivityData["rigidity"])
+    return slip_rate, rigidity
+
+
 @parse_uncertainty.add('simpleFaultGeometryAbsolute')
 def simpleGeom(utype, node, filename):
     if hasattr(node, 'simpleFaultGeometry'):
@@ -263,6 +271,13 @@ def _incMFD_absolute(utype, source, value):
                                       occurrence_rates=occur_rates))
 
 
+@apply_uncertainty.add('truncatedGRFromSlipAbsolute')
+def _trucMFDFromSlip_absolute(utype, source, value):
+    slip_rate, rigidity = value
+    source.modify('adjust_mfd_from_slip', dict(slip_rate=slip_rate,
+                                               rigidity=rigidity))
+
+
 # ######################### apply_uncertainties ########################### #
 
 def apply_uncertainties(bset_values, src_group):
@@ -305,24 +320,47 @@ def apply_uncertainties(bset_values, src_group):
         sg.sources.extend(srcs)
     return sg
 
-
-# ######################### branches and branchsets ######################## #
+# ######################### sampling ######################## #
 
 
-def sample(weighted_objects, num_samples, seed):
+def random(size, seed, sampling_method='early_weights'):
     """
-    Take random samples of a sequence of weighted objects
+    :param size: size of the returned array (integer or pair of integers)
+    :param seed: random seed
+    :param sampling_method: 'early_weights', 'early_latin', ...
+    :returns: an array of floats in the range 0..1
 
-    :param weighted_objects:
-        A finite sequence of objects with a `.weight` attribute.
-        The weights must sum up to 1.
-    :param num_samples:
-        The number of samples to return
-    :param seed:
-        A random seed
-    :return:
-        A subsequence of the original sequence with `num_samples` elements
+    You can compare montecarlo sampling with latin square sampling with
+    the following code:
+
+    import matplotlib.pyplot as plt
+    samples, seed = 10, 42
+    x, y = random((samples, 2), seed, 'early_latin').T
+    plt.xlim([0, 1])
+    plt.ylim([0, 1])
+    plt.scatter(x, y, color='green')  # points on a latin square
+    x, y = random((samples, 2), seed, 'early_weights').T
+    plt.scatter(x, y, color='red')  # points NOT on a latin square
+    for x in numpy.arange(0, 1, 1/samples):
+        for y in numpy.arange(0, 1, 1/samples):
+            plt.axvline(x)
+            plt.axhline(y)
+    plt.show()
     """
+    numpy.random.seed(seed)
+    xs = numpy.random.uniform(size=size)
+    if sampling_method.endswith('latin'):
+        # https://zmurchok.github.io/2019/03/15/Latin-Hypercube-Sampling.html
+        try:
+            s, d = size
+        except TypeError:  # cannot unpack non-iterable int object
+            return (numpy.argsort(xs) + xs) / size
+        for i in range(d):
+            xs[:, i] = (numpy.argsort(xs[:, i]) + xs[:, i]) / s
+    return xs
+
+
+def _cdf(weighted_objects):
     weights = []
     for obj in weighted_objects:
         w = obj.weight
@@ -330,12 +368,64 @@ def sample(weighted_objects, num_samples, seed):
             weights.append(w)
         else:
             weights.append(w['weight'])
-    numpy.random.seed(seed)
-    idxs = numpy.random.choice(len(weights), num_samples, p=weights)
+    return numpy.cumsum(weights)
+
+
+def sample(weighted_objects, probabilities, sampling_method):
+    """
+    Take random samples of a sequence of weighted objects
+
+    :param weighted_objects:
+        A finite sequence of N objects with a `.weight` attribute.
+        The weights must sum up to 1.
+    :param probabilities:
+        An array of S random numbers in the range 0..1
+    :return:
+        A list of S objects extracted randomly
+    """
+    if sampling_method.startswith('early'):  # consider the weights
+        idxs = numpy.searchsorted(_cdf(weighted_objects), probabilities)
+    elif sampling_method.startswith('late'):
+        n = len(weighted_objects)  # consider all weights equal
+        idxs = numpy.searchsorted(numpy.arange(1/n, 1, 1/n), probabilities)
     # NB: returning an array would break things
     return [weighted_objects[idx] for idx in idxs]
 
 
+Weighted = collections.namedtuple('Weighted', 'object weight')
+
+
+# used in notebooks for teaching, not in the engine
+def random_sample(branchsets, num_samples, seed, sampling_method):
+    """
+    >>> bsets = [[('X', .4), ('Y', .6)], [('A', .2), ('B', .3), ('C', .5)]]
+    >>> paths = random_sample(bsets, 100, 42, 'early_weights')
+    >>> collections.Counter(paths)
+    Counter({'YC': 26, 'XC': 24, 'YB': 17, 'XA': 13, 'YA': 10, 'XB': 10})
+
+    >>> paths = random_sample(bsets, 100, 42, 'late_weights')
+    >>> collections.Counter(paths)
+    Counter({'XA': 20, 'YA': 18, 'XB': 17, 'XC': 15, 'YB': 15, 'YC': 15})
+
+    >>> paths = random_sample(bsets, 100, 42, 'early_latin')
+    >>> collections.Counter(paths)
+    Counter({'YC': 31, 'XC': 19, 'YB': 17, 'XB': 13, 'YA': 12, 'XA': 8})
+
+    >>> paths = random_sample(bsets, 100, 45, 'late_latin')
+    >>> collections.Counter(paths)
+    Counter({'YC': 18, 'XA': 18, 'XC': 16, 'YA': 16, 'XB': 16, 'YB': 16})
+    """
+    probs = random((num_samples, len(branchsets)), seed, sampling_method)
+    arr = numpy.zeros((num_samples, len(branchsets)), object)
+    for b, bset in enumerate(branchsets):
+        arr[:, b] = sample([Weighted(*it) for it in bset], probs[:, b],
+                           sampling_method)
+    return [''.join(w.object for w in row) for row in arr]
+
+
+# ######################### branches and branchsets ######################## #
+
+
 class Branch(object):
     """
     Branch object, represents a ``<logicTreeBranch />`` element.
@@ -409,6 +499,8 @@ class BranchSet(object):
         characteristicFaultGeometryAbsolute
             Replaces the complex fault geometry surface of a given source with
             the values provided
+        truncatedGRFromSlipAbsolute
+            Updates a TruncatedGR using a slip rate and a rigidity
 
     :param filters:
         Dictionary, a set of filters to specify which sources should
@@ -430,28 +522,35 @@ class BranchSet(object):
             Stable Shallow Crust, etc.) the uncertainty applies to. This
             filter is required for all branchsets in GMPE logic tree.
     """
-    def __init__(self, uncertainty_type, filters=None, collapsed=False):
-        self.branches = []
+    def __init__(self, uncertainty_type, ordinal=0, filters=None,
+                 collapsed=False):
         self.uncertainty_type = uncertainty_type
+        self.ordinal = ordinal
         self.filters = filters or {}
         self.collapsed = collapsed
+        self.branches = []
 
-    def sample(self, seed):
+    def sample(self, probabilities, sampling_method):
         """
-        Return a list of branches.
-
-        :param seed: the seed used for the sampling
+        :param num_samples: the number of samples
+        :param probabilities: (Ns, Nb) random numbers in the range 0..1
+        :param sampling_method: the sampling method used
+        :returns: a list of num_samples lists of branches
         """
-        branchset = self
-        branches = []
-        while branchset is not None:
-            if branchset.collapsed:
-                branch = branchset.branches[0]
-            else:
-                [branch] = sample(branchset.branches, 1, seed)
-            branches.append(branch)
-            branchset = branch.bset
-        return branches
+        out = []
+        for probs in probabilities:  # probs has a value for each branchset
+            branchset = self
+            branches = []
+            while branchset is not None:
+                if branchset.collapsed:
+                    branch = branchset.branches[0]
+                else:
+                    x = probs[branchset.ordinal]
+                    [branch] = sample(branchset.branches, [x], sampling_method)
+                branches.append(branch)
+                branchset = branch.bset
+            out.append(branches)
+        return out
 
     def enumerate_paths(self):
         """
