@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2020 GEM Foundation
+# Copyright (C) 2012-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -20,11 +20,12 @@
 Module :mod:`openquake.hazardlib.site` defines :class:`Site`.
 """
 import numpy
+from scipy.spatial import distance
 from shapely import geometry
 from openquake.baselib.general import (
     split_in_blocks, not_equal, get_duplicates)
 from openquake.hazardlib.geo.utils import (
-    fix_lon, cross_idl, _GeographicObjects, geohash)
+    fix_lon, cross_idl, _GeographicObjects, geohash, spherical_to_cartesian)
 from openquake.hazardlib.geo.mesh import Mesh
 
 U32LIMIT = 2 ** 32
@@ -123,7 +124,6 @@ site_param_dt = {
     'z1pt4': numpy.float64,
     'backarc': numpy.bool,
     'xvf': numpy.float64,
-    'backarc_distance': numpy.float64,
 
     # Parameters for site amplification
     'ampcode': ampcode_dt,
@@ -131,19 +131,32 @@ site_param_dt = {
     'ec8_p18': (numpy.string_, 2),
     'h800': numpy.float64,
     'geology': (numpy.string_, 20),
+    'amplfactor': numpy.float64,
 
-    # parameters for geotechnic hazard
-    'liquefaction_susceptibility': numpy.int16,
-    'landsliding_susceptibility': numpy.int16,
+    # parameters for secondary perils
+    'friction_mid': numpy.float64,
+    'cohesion_mid': numpy.float64,
+    'saturation': numpy.float64,
+    'dry_density': numpy.float64,
+    'Fs': numpy.float64,
+    'crit_accel': numpy.float64,
+    'unit': (numpy.string_, 5),
+    'liq_susc_cat': (numpy.string_, 2),
     'dw': numpy.float64,
     'yield_acceleration': numpy.float64,
     'slope': numpy.float64,
+    'gwd': numpy.float64,
     'cti': numpy.float64,
     'dc': numpy.float64,
     'dr': numpy.float64,
     'dwb': numpy.float64,
     'hwater': numpy.float64,
-    'precip': numpy.float64
+    'precip': numpy.float64,
+    'fpeak': numpy.float64,
+
+    # other parameters
+    'custom_site_id': numpy.uint32,
+    'region': numpy.uint32
 }
 
 
@@ -253,10 +266,9 @@ class SiteCollection(object):
         return self
 
     def _set(self, param, value):
-        # param comes from the file site_model.xml file which usually contains
-        # a lot of parameters; the parameters that are not required are ignored
-        if param in self.array.dtype.names:  # is required
-            self.array[param] = value
+        if param not in self.array.dtype.names:
+            self.add_col(param, site_param_dt[param])
+        self.array[param] = value
 
     xyz = Mesh.xyz
 
@@ -276,7 +288,18 @@ class SiteCollection(object):
         new.complete = self.complete
         return new
 
-    def add_col(self, colname, dtype):
+    def reduce(self, nsites):
+        """
+        :returns: a filtered SiteCollection with around nsites (if nsites<=N)
+        """
+        N = len(self.complete)
+        n = N // nsites + 1
+        if n == 1:
+            return self
+        sids, = numpy.where(self.complete.sids % n == 0)
+        return self.filtered(sids)
+
+    def add_col(self, colname, dtype, values=None):
         """
         Add a column to the underlying array
         """
@@ -286,6 +309,8 @@ class SiteCollection(object):
         arr = numpy.zeros(len(self), dtlist)
         for name in names:
             arr[name] = self.array[name]
+        if values is not None:
+            arr[colname] = values
         self.array = arr
 
     def make_complete(self):
@@ -306,6 +331,19 @@ class SiteCollection(object):
             idx = 0
         return self.filtered([self.sids[idx]])
 
+    # used for debugging purposes
+    def get_cdist(self, rec_or_loc):
+        """
+        :param rec_or_loc: a record with field 'hypo' or a Point instance
+        :returns: array of N euclidean distances from rec['hypo']
+        """
+        try:
+            lon, lat, dep = rec_or_loc['hypo']
+        except TypeError:
+            lon, lat, dep = rec_or_loc.x, rec_or_loc.y, rec_or_loc.z
+        xyz = spherical_to_cartesian(lon, lat, dep).reshape(1, 3)
+        return distance.cdist(self.xyz, xyz)[:, 0]
+
     def __init__(self, sites):
         """
         Build a complete SiteCollection from a list of Site objects
@@ -369,17 +407,15 @@ class SiteCollection(object):
         for seq in split_in_blocks(range(len(self)), hint or 1):
             sc = SiteCollection.__new__(SiteCollection)
             sc.array = self.array[numpy.array(seq, int)]
+            sc.complete = self
             tiles.append(sc)
         return tiles
 
-    def split(self, location, distance):
+    def count_close(self, location, distance):
         """
-        :returns: (close_sites, far_sites)
+        :returns: the number of sites within the distance from the location
         """
-        if distance is None:  # all close
-            return self, None
-        close = location.distance_to_mesh(self) < distance
-        return self.filter(close), self.filter(~close)
+        return (self.get_cdist(location) < distance).sum()
 
     def __iter__(self):
         """
@@ -472,7 +508,7 @@ class SiteCollection(object):
 
     def geohash(self, length):
         """
-        :param length: length of the geohash
+        :param length: length of the geohash in the range 1..8
         :returns: an array of N geohashes, one per site
         """
         lst = [geohash(lon, lat, length)
@@ -481,7 +517,7 @@ class SiteCollection(object):
 
     def num_geohashes(self, length):
         """
-        :param length: length of the geohash
+        :param length: length of the geohash in the range 1..8
         :returns: number of distinct geohashes in the site collection
         """
         return len(numpy.unique(self.geohash(length)))
