@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2016-2020 GEM Foundation
+# Copyright (C) 2016-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -98,6 +98,8 @@ def create_job(db, datadir):
         the job ID
     """
     calc_id = get_calc_id(db, datadir) + 1
+    # HACK: just created jobs should not have is_running=1, but we
+    # need that to make views_test.py happy on Jenkins :-(
     job = dict(id=calc_id, is_running=1, description='just created',
                user_name=getpass.getuser(), calculation_mode='to be set',
                ds_calc_dir=os.path.join('%s/calc_%s' % (datadir, calc_id)))
@@ -237,8 +239,9 @@ def list_outputs(db, job_id, full=True):
                 break
             out.append('%4d | %s' % (o.id, o.display_name))
         if truncated:
-            out.append('Some outputs where not shown. You can see the full '
-                       'list with the command\n`oq engine --list-outputs`')
+            out.append(
+                'Some outputs were not shown. You can see the full list '
+                f'with the command\n`oq engine --list-outputs {job_id}`')
     return out
 
 
@@ -257,13 +260,11 @@ def get_outputs(db, job_id):
 DISPLAY_NAME = {
     'asset_risk': 'Exposure + Risk',
     'gmf_data': 'Ground Motion Fields',
-    'dmg_by_asset': 'Average Asset Damages',
-    'dmg_by_event': 'Aggregate Event Damages',
-    'losses_by_asset': 'Average Asset Losses',
-    'losses_by_event': 'Aggregate Event Losses',
-    'events': 'Events',
-    'damages-rlzs': 'Asset Damage Distribution',
+    'damages-rlzs': 'Asset Damage Distributions',
     'damages-stats': 'Asset Damage Statistics',
+    'dmg_by_event': 'Aggregate Event Damages',
+    'agg_loss_table': 'Aggregate Event Losses',
+    'events': 'Events',
     'avg_losses-rlzs': 'Average Asset Losses',
     'avg_losses-stats': 'Average Asset Losses Statistics',
     'loss_curves-rlzs': 'Asset Loss Curves',
@@ -276,10 +277,7 @@ DISPLAY_NAME = {
     'agg_losses-stats': 'Aggregate Losses Statistics',
     'agg_risk': 'Total Risk',
     'agglosses': 'Aggregate Asset Losses',
-    'tot_losses-rlzs': 'Total Losses',
-    'tot_losses-stats': 'Total Losses Statistics',
-    'tot_curves-rlzs': 'Total Loss Curves',
-    'tot_curves-stats': 'Total Loss Curves Statistics',
+    'avg_gmf': 'Average Ground Motion Field',
     'bcr-rlzs': 'Benefit Cost Ratios',
     'bcr-stats': 'Benefit Cost Ratios Statistics',
     'ruptures': 'Earthquake Ruptures',
@@ -287,8 +285,8 @@ DISPLAY_NAME = {
     'hmaps': 'Hazard Maps',
     'uhs': 'Uniform Hazard Spectra',
     'disagg': 'Disaggregation Outputs',
-    'disagg_by_src': 'Disaggregation by Source',
     'realizations': 'Realizations',
+    'src_loss_table': 'Source Loss Table',
     'fullreport': 'Full Report',
     'input': 'Input Files'
 }
@@ -345,7 +343,18 @@ def del_calc(db, job_id, user, force=False):
     dependent = db(
         "SELECT id FROM job WHERE hazard_calculation_id=?x "
         "AND status != 'deleted'", job_id)
-    if not force and dependent:
+    job_ids = [dep.id for dep in dependent]
+    if not force and job_id in job_ids:  # jobarray
+        err = []
+        for jid in job_ids:
+            res = del_calc(db, jid, user, force=True)
+            if "error" in res:
+                err.append(res["error"])
+        if err:
+            return {"error": ' '.join(err)}
+        else:
+            return {"success": 'children_of_%s' % job_id}
+    elif not force and dependent:
         return {"error": 'Cannot delete calculation %d: there '
                 'are calculations '
                 'dependent from it: %s' % (job_id, [j.id for j in dependent])}
